#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields

from polyaxon.constants.globals import PLATFORM_DIST_CE
from polyaxon.schemas.api.log_handler import LogHandlerSchema
from polyaxon.schemas.cli.checks_config import ChecksConfig, ChecksSchema


class CliSchema(ChecksSchema):
    current_version = fields.Str(allow_none=True)
    installation = fields.Dict(allow_none=True)
    compatibility = fields.Dict(allow_none=True)
    log_handler = fields.Nested(LogHandlerSchema, allow_none=True)

    @staticmethod
    def schema_config():
        return CliConfig


class CliConfig(ChecksConfig):
    SCHEMA = CliSchema
    IDENTIFIER = "cli"
    INTERVAL = 30 * 60

    def __init__(
        self,
        current_version=None,
        installation=None,
        compatibility=None,
        log_handler=None,
        last_check=None,
    ):
        self.current_version = current_version
        self.installation = installation
        self.compatibility = compatibility
        self.log_handler = log_handler
        super().__init__(last_check=last_check)

    @property
    def min_version(self):
        if not self.compatibility or "cli" not in self.compatibility:
            return None
        cli_version = self.compatibility["cli"] or {}
        return cli_version.get("min")

    @property
    def latest_version(self):
        if not self.compatibility or "cli" not in self.compatibility:
            return None
        cli_version = self.compatibility["cli"] or {}
        return cli_version.get("latest")

    @property
    def is_ce(self):
        return self.installation and self.installation.get("dist") == PLATFORM_DIST_CE



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from datetime import datetime
from typing import Dict, List, Optional

from polyaxon import settings
from polyaxon.exceptions import PolyaxonCompilerError
from polyaxon.polyaxonfile import CompiledOperationSpecification
from polyaxon.polyflow import V1CloningKind, V1CompiledOperation, V1Operation, V1RunKind
from polyaxon.polypod.compiler.config import PolypodConfig
from polyaxon.polypod.compiler.contexts import (
    resolve_contexts,
    resolve_globals_contexts,
)
from polyaxon.polypod.compiler.lineage import resolve_artifacts_lineage


class BaseResolver:
    KINDS = {
        V1RunKind.JOB,
        V1RunKind.SERVICE,
        V1RunKind.MPIJOB,
        V1RunKind.TFJOB,
        V1RunKind.PYTORCHJOB,
        V1RunKind.MXJOB,
        V1RunKind.XGBJOB,
        V1RunKind.NOTIFIER,
        V1RunKind.CLEANER,
        V1RunKind.TUNER,
        V1RunKind.WATCHDOG,
    }

    def __init__(
        self,
        run,
        compiled_operation: V1CompiledOperation,
        owner_name: str,
        project_name: str,
        project_uuid: str,
        run_name: str,
        run_uuid: str,
        run_path: str,
        params: Optional[Dict],
        created_at: datetime = None,
        compiled_at: datetime = None,
        cloning_kind: V1CloningKind = None,
        original_uuid: str = None,
        is_independent: bool = True,
        eager: bool = False,
    ):
        if not compiled_operation:
            raise PolyaxonCompilerError("A run spec is required for resolution.")
        self.run = run
        self.compiled_operation = compiled_operation
        self.owner_name = owner_name
        self.project_name = project_name
        self.project_uuid = project_uuid
        self.project_uuid = project_uuid or project_name
        self.run_name = run_name
        self.run_uuid = run_uuid or run_name
        self.run_path = run_path
        self.params = params or {}
        self.connection_by_names = {}
        self.namespace = None
        self.artifacts_store = None
        self.secrets = None
        self.config_maps = None
        self.polyaxon_sidecar = None
        self.polyaxon_init = None
        self.iteration = None
        self.agent_config = None
        self.globals = {}
        self.artifacts = []
        self.created_at = created_at
        self.compiled_at = compiled_at
        self.schedule_at = None
        self.started_at = None
        self.finished_at = None
        self.duration = None
        self.eager = eager
        self.cloning_kind = cloning_kind
        self.original_uuid = original_uuid
        self.is_independent = is_independent
        self._param_spec = {}

    @classmethod
    def is_valid(cls, compiled_operation: V1CompiledOperation):
        compiled_operation.validate_build()
        run_kind = compiled_operation.get_run_kind()
        if run_kind not in cls.KINDS:
            raise PolyaxonCompilerError(
                "Resolver Error. "
                "Specification with run kind: {} "
                "is not supported in this deployment version.".format(run_kind)
            )

    @property
    def param_spec(self):
        return self._param_spec

    def resolve_edges(self):
        pass

    def resolve_globals_contexts(self):
        self.globals = resolve_globals_contexts(
            namespace=self.namespace,
            owner_name=self.owner_name,
            project_name=self.project_name,
            project_uuid=self.project_uuid,
            run_uuid=self.run_uuid,
            run_name=self.run_name,
            run_path=self.run_path,
            iteration=self.iteration,
            created_at=self.created_at,
            compiled_at=self.compiled_at,
            schedule_at=self.schedule_at,
            started_at=self.started_at,
            finished_at=self.finished_at,
            duration=self.duration,
            cloning_kind=self.cloning_kind,
            original_uuid=self.original_uuid,
            is_independent=self.is_independent,
        )

    def resolve_params(self):
        pass

    def apply_params(self, should_be_resolved: bool = True):
        self.compiled_operation = CompiledOperationSpecification.apply_params(
            config=self.compiled_operation,
            params=self.params,
            context=self.globals,
        )
        self._param_spec = CompiledOperationSpecification.calculate_context_spec(
            config=self.compiled_operation,
            contexts=self.globals,
            should_be_resolved=should_be_resolved,
        )

    def resolve_connections_params(self):
        self.compiled_operation = (
            CompiledOperationSpecification.apply_run_connections_params(
                config=self.compiled_operation,
                artifact_store=self.agent_config.artifacts_store.name
                if self.agent_config
                else None,
                contexts=self.globals,
                param_spec=self.param_spec,
            )
        )

    def resolve_presets(self):
        pass

    def resolve_agent(self):
        if settings.AGENT_CONFIG:
            self.agent_config = settings.AGENT_CONFIG.clone()

    def apply_operation_contexts(self):
        try:
            self.compiled_operation = (
                CompiledOperationSpecification.apply_operation_contexts(
                    self.compiled_operation,
                    param_spec=self.param_spec,
                    contexts=self.globals,
                )
            )
        except Exception as e:
            raise PolyaxonCompilerError(
                "Could not apply run context, error: {}".format(repr(e))
            )

    def resolve_io(self):
        pass

    def resolve_access(self):
        pass

    def resolve_connections(self):
        polypod_config = PolypodConfig()
        polypod_config.resolve(
            compiled_operation=self.compiled_operation, agent_config=self.agent_config
        )
        self.polyaxon_sidecar = polypod_config.polyaxon_sidecar
        self.polyaxon_init = polypod_config.polyaxon_init
        self.namespace = polypod_config.namespace
        self.secrets = polypod_config.secrets
        self.config_maps = polypod_config.config_maps
        self.connection_by_names = polypod_config.connection_by_names
        self.artifacts_store = polypod_config.artifacts_store

    def resolve_init_refs(self):
        pass

    def clean_init_refs(self):
        pass

    def resolve_actions(self):
        pass

    def resolve_artifacts_lineage(self):
        self.artifacts += resolve_artifacts_lineage(
            owner_name=self.owner_name,
            project_name=self.project_name,
            project_uuid=self.project_uuid,
            run_name=self.run_name,
            run_path=self.run_path,
            run_uuid=self.run_uuid,
            param_spec=self.param_spec,
            compiled_operation=self.compiled_operation,
            connection_by_names=self.connection_by_names,
            artifacts_store=self.artifacts_store,
        )

    def _resolve_contexts(self):
        return resolve_contexts(
            namespace=self.namespace,
            owner_name=self.owner_name,
            project_name=self.project_name,
            project_uuid=self.project_uuid,
            run_uuid=self.run_uuid,
            run_name=self.run_name,
            run_path=self.run_path,
            compiled_operation=self.compiled_operation,
            artifacts_store=self.artifacts_store,
            connection_by_names=self.connection_by_names,
            iteration=self.iteration,
            created_at=self.created_at,
            compiled_at=self.compiled_at,
            schedule_at=self.schedule_at,
            started_at=self.started_at,
            finished_at=self.finished_at,
            duration=self.duration,
            cloning_kind=self.cloning_kind,
            original_uuid=self.original_uuid,
            is_independent=self.is_independent,
        )

    def _apply_runtime_contexts(self):
        contexts = self._resolve_contexts()
        return CompiledOperationSpecification.apply_runtime_contexts(
            self.compiled_operation, contexts=contexts
        )

    def _apply_pipeline_contexts(self):
        return self.compiled_operation

    def _should_skip_runtime_resolution(self):
        return False

    def apply_runtime_contexts(self):
        if self._should_skip_runtime_resolution():
            return
        if self.compiled_operation.has_pipeline:
            self.compiled_operation = self._apply_pipeline_contexts()
        else:
            self.compiled_operation = self._apply_runtime_contexts()

    def resolve_state(self):
        pass

    def persist_state(self):
        pass

    def resolve_build(self):
        raise PolyaxonCompilerError(
            "Build resolution is not supported in this Polyaxon distribution"
        )

    def resolve_hooks(self) -> List[V1Operation]:
        raise PolyaxonCompilerError(
            "Hooks resolution is not supported in this Polyaxon distribution"
        )

    def resolve(self) -> V1CompiledOperation:
        self.resolve_edges()
        self.resolve_globals_contexts()
        self.resolve_params()
        self.apply_params()
        self.resolve_presets()
        self.resolve_agent()
        self.resolve_connections_params()
        self.apply_operation_contexts()
        self.resolve_io()
        self.resolve_init_refs()
        self.resolve_access()
        self.resolve_connections()
        self.resolve_artifacts_lineage()
        self.clean_init_refs()
        self.resolve_state()
        self.apply_runtime_contexts()
        self.persist_state()
        return self.compiled_operation



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    Polyaxon SDKs and REST API specification.

    Polyaxon SDKs and REST API specification.  # noqa: E501

    The version of the OpenAPI document: 1.16.0
    Contact: contact@polyaxon.com
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from polyaxon_sdk.configuration import Configuration


class V1Init(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'artifacts': 'V1ArtifactsType',
        'git': 'V1GitType',
        'dockerfile': 'V1DockerfileType',
        'file': 'V1FileType',
        'artifact_ref': 'str',
        'model_ref': 'str',
        'connection': 'str',
        'path': 'str',
        'container': 'V1Container'
    }

    attribute_map = {
        'artifacts': 'artifacts',
        'git': 'git',
        'dockerfile': 'dockerfile',
        'file': 'file',
        'artifact_ref': 'artifactRef',
        'model_ref': 'modelRef',
        'connection': 'connection',
        'path': 'path',
        'container': 'container'
    }

    def __init__(self, artifacts=None, git=None, dockerfile=None, file=None, artifact_ref=None, model_ref=None, connection=None, path=None, container=None, local_vars_configuration=None):  # noqa: E501
        """V1Init - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._artifacts = None
        self._git = None
        self._dockerfile = None
        self._file = None
        self._artifact_ref = None
        self._model_ref = None
        self._connection = None
        self._path = None
        self._container = None
        self.discriminator = None

        if artifacts is not None:
            self.artifacts = artifacts
        if git is not None:
            self.git = git
        if dockerfile is not None:
            self.dockerfile = dockerfile
        if file is not None:
            self.file = file
        if artifact_ref is not None:
            self.artifact_ref = artifact_ref
        if model_ref is not None:
            self.model_ref = model_ref
        if connection is not None:
            self.connection = connection
        if path is not None:
            self.path = path
        if container is not None:
            self.container = container

    @property
    def artifacts(self):
        """Gets the artifacts of this V1Init.  # noqa: E501


        :return: The artifacts of this V1Init.  # noqa: E501
        :rtype: V1ArtifactsType
        """
        return self._artifacts

    @artifacts.setter
    def artifacts(self, artifacts):
        """Sets the artifacts of this V1Init.


        :param artifacts: The artifacts of this V1Init.  # noqa: E501
        :type artifacts: V1ArtifactsType
        """

        self._artifacts = artifacts

    @property
    def git(self):
        """Gets the git of this V1Init.  # noqa: E501


        :return: The git of this V1Init.  # noqa: E501
        :rtype: V1GitType
        """
        return self._git

    @git.setter
    def git(self, git):
        """Sets the git of this V1Init.


        :param git: The git of this V1Init.  # noqa: E501
        :type git: V1GitType
        """

        self._git = git

    @property
    def dockerfile(self):
        """Gets the dockerfile of this V1Init.  # noqa: E501


        :return: The dockerfile of this V1Init.  # noqa: E501
        :rtype: V1DockerfileType
        """
        return self._dockerfile

    @dockerfile.setter
    def dockerfile(self, dockerfile):
        """Sets the dockerfile of this V1Init.


        :param dockerfile: The dockerfile of this V1Init.  # noqa: E501
        :type dockerfile: V1DockerfileType
        """

        self._dockerfile = dockerfile

    @property
    def file(self):
        """Gets the file of this V1Init.  # noqa: E501


        :return: The file of this V1Init.  # noqa: E501
        :rtype: V1FileType
        """
        return self._file

    @file.setter
    def file(self, file):
        """Sets the file of this V1Init.


        :param file: The file of this V1Init.  # noqa: E501
        :type file: V1FileType
        """

        self._file = file

    @property
    def artifact_ref(self):
        """Gets the artifact_ref of this V1Init.  # noqa: E501


        :return: The artifact_ref of this V1Init.  # noqa: E501
        :rtype: str
        """
        return self._artifact_ref

    @artifact_ref.setter
    def artifact_ref(self, artifact_ref):
        """Sets the artifact_ref of this V1Init.


        :param artifact_ref: The artifact_ref of this V1Init.  # noqa: E501
        :type artifact_ref: str
        """

        self._artifact_ref = artifact_ref

    @property
    def model_ref(self):
        """Gets the model_ref of this V1Init.  # noqa: E501


        :return: The model_ref of this V1Init.  # noqa: E501
        :rtype: str
        """
        return self._model_ref

    @model_ref.setter
    def model_ref(self, model_ref):
        """Sets the model_ref of this V1Init.


        :param model_ref: The model_ref of this V1Init.  # noqa: E501
        :type model_ref: str
        """

        self._model_ref = model_ref

    @property
    def connection(self):
        """Gets the connection of this V1Init.  # noqa: E501


        :return: The connection of this V1Init.  # noqa: E501
        :rtype: str
        """
        return self._connection

    @connection.setter
    def connection(self, connection):
        """Sets the connection of this V1Init.


        :param connection: The connection of this V1Init.  # noqa: E501
        :type connection: str
        """

        self._connection = connection

    @property
    def path(self):
        """Gets the path of this V1Init.  # noqa: E501


        :return: The path of this V1Init.  # noqa: E501
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path):
        """Sets the path of this V1Init.


        :param path: The path of this V1Init.  # noqa: E501
        :type path: str
        """

        self._path = path

    @property
    def container(self):
        """Gets the container of this V1Init.  # noqa: E501


        :return: The container of this V1Init.  # noqa: E501
        :rtype: V1Container
        """
        return self._container

    @container.setter
    def container(self, container):
        """Sets the container of this V1Init.


        :param container: The container of this V1Init.  # noqa: E501
        :type container: V1Container
        """

        self._container = container

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1Init):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1Init):
            return True

        return self.to_dict() != other.to_dict()



# !/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, List

from polyaxon.polyboard.processors.events_processors import metrics_dict_to_list

try:
    import psutil
except ImportError:
    psutil = None


def can_log_psutil_resources():
    return psutil is not None


def query_psutil() -> Dict:
    results = {}
    try:
        # psutil <= 5.6.2 did not have getloadavg:
        if hasattr(psutil, "getloadavg"):
            results["load"] = psutil.getloadavg()[0]
        else:
            # Do not log an empty metric
            pass
    except OSError:
        pass
    vm = psutil.virtual_memory()
    results["cpu"] = psutil.cpu_percent(interval=None)
    results["memory"] = vm.percent
    return results


def get_psutils_metrics() -> List:
    return metrics_dict_to_list(query_psutil())



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Any

from marshmallow import fields

from polyaxon.k8s import k8s_schemas
from polyaxon.schemas.base import BaseCamelSchema, BaseConfig
from polyaxon.schemas.fields.swagger import SwaggerField


class DefaultSchedulingSchema(BaseCamelSchema):
    node_selector = fields.Dict(allow_none=True)
    affinity = SwaggerField(cls=k8s_schemas.V1Affinity, allow_none=True)
    tolerations = fields.List(
        SwaggerField(cls=k8s_schemas.V1Toleration), allow_none=True
    )
    image_pull_secrets = fields.List(fields.Str(), allow_none=True)

    @staticmethod
    def schema_config():
        return V1DefaultScheduling


class V1DefaultScheduling(BaseConfig):
    SCHEMA = DefaultSchedulingSchema
    IDENTIFIER = "default_scheduling"
    REDUCED_ATTRIBUTES = [
        "nodeSelector",
        "affinity",
        "tolerations",
        "imagePullSecrets",
    ]

    def __init__(
        self,
        node_selector=None,
        affinity=None,
        tolerations=None,
        image_pull_secrets=None,
    ):
        self.node_selector = node_selector
        self.affinity = affinity
        self.tolerations = tolerations
        self.image_pull_secrets = image_pull_secrets

    @staticmethod
    def get_service_environment(
        service: Any,
        default_scheduling: "V1DefaultScheduling",
    ) -> "V1Environment":
        from polyaxon.polyflow import V1Environment

        env = V1Environment()
        if service and service.node_selector:
            env.node_selector = service.node_selector
        elif default_scheduling and default_scheduling.node_selector:
            env.node_selector = default_scheduling.node_selector
        if service and service.affinity:
            env.affinity = service.affinity
        elif default_scheduling and default_scheduling.affinity:
            env.affinity = default_scheduling.affinity
        if service and service.tolerations:
            env.tolerations = service.tolerations
        elif default_scheduling and default_scheduling.tolerations:
            env.tolerations = default_scheduling.tolerations
        if service and service.image_pull_secrets:
            env.image_pull_secrets = service.image_pull_secrets
        elif default_scheduling and default_scheduling.image_pull_secrets:
            env.image_pull_secrets = default_scheduling.image_pull_secrets

        return env



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from polyaxon.k8s.async_manager import AsyncK8SManager


async def get_pods(
    k8s_manager: AsyncK8SManager,
    run_uuid: str,
):
    await k8s_manager.setup()
    pods = await k8s_manager.list_pods(
        label_selector=k8s_manager.get_managed_by_polyaxon(run_uuid)
    )
    pods_list = {}
    for pod in pods or []:
        pods_list[
            pod.metadata.name
        ] = k8s_manager.api_client.sanitize_for_serialization(pod)
    return pods_list



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import List, Union

import click

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.schemas.types import V1ConnectionType
from polyaxon.utils.formatting import Printer
from polyaxon.utils.list_utils import to_list


@click.group()
def clean_artifacts():
    pass


def _delete(
    subpath: Union[str, List[str]],
    connection_name: str,
    connection_kind: str,
    is_file: bool,
):
    from polyaxon.fs.fs import get_sync_fs_from_type
    from polyaxon.fs.manager import delete_file_or_dir

    subpath = to_list(subpath, check_none=True)
    connection_type = V1ConnectionType(name=connection_name, kind=connection_kind)
    fs = get_sync_fs_from_type(connection_type=connection_type)
    for sp in subpath:
        delete_file_or_dir(
            fs=fs,
            subpath=sp,
            is_file=is_file,
        )
    Printer.print_success(
        "{} subpath was cleaned, subpath: `{}`".format(connection_kind, subpath)
    )


@clean_artifacts.command()
@click.option("--connection-name", help="The connection name.")
@click.option("-sp", "--subpath", multiple=True, help="The s3 subpath to clean.")
@click.option(
    "--is-file",
    is_flag=True,
    default=False,
    help="whether or not to use the basename of the key.",
)
def s3(connection_name, subpath, is_file):
    """Delete an s3 subpath."""
    _delete(
        subpath=subpath,
        connection_name=connection_name,
        connection_kind=V1ConnectionKind.S3,
        is_file=is_file,
    )


@clean_artifacts.command()
@click.option("--connection-name", help="The connection name.")
@click.option("-sp", "--subpath", multiple=True, help="The gcs subpath to clean.")
@click.option(
    "--is-file",
    is_flag=True,
    default=False,
    help="whether or not to use the basename of the key.",
)
def gcs(connection_name, subpath, is_file):
    """Delete a gcs subpath."""
    _delete(
        subpath=subpath,
        connection_name=connection_name,
        connection_kind=V1ConnectionKind.GCS,
        is_file=is_file,
    )


@clean_artifacts.command()
@click.option("--connection-name", help="The connection name.")
@click.option("-sp", "--subpath", multiple=True, help="The wasb subpath to clean.")
@click.option(
    "--is-file",
    is_flag=True,
    default=False,
    help="whether or not to use the basename of the key.",
)
def wasb(connection_name, subpath, is_file):
    """Delete a wasb path context."""
    _delete(
        subpath=subpath,
        connection_name=connection_name,
        connection_kind=V1ConnectionKind.WASB,
        is_file=is_file,
    )


@clean_artifacts.command()
@click.option("--connection-name", help="The connection name.")
@click.option("-sp", "--subpath", multiple=True, help="The volume subpath to clean.")
@click.option(
    "--is-file",
    is_flag=True,
    default=False,
    help="whether or not to use the basename of the key.",
)
def volume_claim(connection_name, subpath, is_file):
    """Delete a volume path context."""
    _delete(
        subpath=subpath,
        connection_name=connection_name,
        connection_kind=V1ConnectionKind.VOLUME_CLAIM,
        is_file=is_file,
    )


@clean_artifacts.command()
@click.option("--connection-name", help="The connection name.")
@click.option("-sp", "--subpath", multiple=True, help="The host subpath to clean.")
@click.option(
    "--is-file",
    is_flag=True,
    default=False,
    help="whether or not to use the basename of the key.",
)
def host_path(connection_name, subpath, is_file):
    """Delete a host path context."""
    _delete(
        subpath=subpath,
        connection_name=connection_name,
        connection_kind=V1ConnectionKind.HOST_PATH,
        is_file=is_file,
    )



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from polyaxon.api import STREAMS_V1_LOCATION
from polyaxon.proxies.schemas.base import get_config

GUNICORN_OPTIONS = """
location / {{
    proxy_pass http://polyaxon;
    proxy_http_version 1.1;
    proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_intercept_errors {intercept_errors};
}}
"""


def get_gunicorn_config():
    return get_config(options=GUNICORN_OPTIONS, indent=0, intercept_errors="off")


K8S_AUTH_OPTIONS = """
location {app}k8s/auth/ {{
    proxy_method      GET;
    proxy_pass http://polyaxon;
    proxy_http_version 1.1;
    proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header X-Origin-URI $request_uri;
    proxy_intercept_errors {intercept_errors};
}}
"""


def get_k8s_auth_config():
    return get_config(
        options=K8S_AUTH_OPTIONS,
        app=STREAMS_V1_LOCATION,
        indent=0,
        intercept_errors="off",
    )



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import Dict, List

from polyaxon.polyflow import V1Notification, V1SchedulingPolicy, V1Termination


def set_termination(custom_object: Dict, termination: V1Termination) -> Dict:
    if not termination:
        return custom_object

    termination_spec = {}
    if termination.max_retries:
        termination_spec["backoffLimit"] = termination.max_retries
    if termination.timeout:
        termination_spec["activeDeadlineSeconds"] = termination.timeout
    if termination.ttl:
        termination_spec["ttlSecondsAfterFinished"] = termination.ttl

    custom_object["termination"] = termination_spec
    return custom_object


def set_collect_logs(custom_object: Dict, collect_logs: bool) -> Dict:
    if collect_logs is None:
        collect_logs = False

    custom_object["collectLogs"] = collect_logs
    return custom_object


def set_sync_statuses(custom_object: Dict, sync_statuses: bool) -> Dict:
    if sync_statuses is None:
        sync_statuses = False

    custom_object["syncStatuses"] = sync_statuses
    return custom_object


def set_notify(custom_object: Dict, notifications: List[V1Notification]) -> Dict:
    if notifications is None:
        notifications = []

    custom_object["notifications"] = [n.to_operator() for n in notifications]
    return custom_object


def set_clean_pod_policy(template_spec: Dict, clean_pod_policy: str) -> Dict:
    if not clean_pod_policy:
        # Sets default clean pod policy
        clean_pod_policy = "All"

    template_spec["cleanPodPolicy"] = clean_pod_policy.capitalize()
    return template_spec


def set_scheduling_policy(
    template_spec: Dict, scheduling_policy: V1SchedulingPolicy = None
) -> Dict:
    if not scheduling_policy:
        return template_spec

    template_spec["schedulingPolicy"] = scheduling_policy.to_light_dict()
    return template_spec



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Optional

from polyaxon import settings
from polyaxon.connections.reader import get_connection_context_path, get_connection_type
from polyaxon.env_vars.getters import get_artifacts_store_name
from polyaxon.exceptions import PolyaxonStoresException
from polyaxon.schemas.types import V1ConnectionType
from polyaxon_sdk import V1ConnectionKind


def validate_store(connection_type: V1ConnectionType):
    if not connection_type or not connection_type.is_artifact:
        raise PolyaxonStoresException("An artifact store type was not provided.")


def get_artifacts_connection_type() -> Optional[V1ConnectionType]:
    store_name = get_artifacts_store_name()
    if store_name:
        return get_connection_type(store_name)
    if settings.AGENT_CONFIG:
        return settings.AGENT_CONFIG.artifacts_store
    return None


def _get_fs_from_type(
    connection_type: V1ConnectionType,
    asynchronous: bool = False,
    use_listings_cache: bool = False,
    **kwargs
):
    connection_name = connection_type.name if connection_type else None
    context_path = get_connection_context_path(name=connection_name)

    # We assume that `None` refers to local store as well
    if not connection_type or connection_type.kind in {
        V1ConnectionKind.VOLUME_CLAIM,
        V1ConnectionKind.HOST_PATH,
    }:
        from fsspec.implementations.local import LocalFileSystem

        return LocalFileSystem(
            auto_mkdir=kwargs.get("auto_mkdir", True),
            use_listings_cache=use_listings_cache,
        )
    if connection_type.kind == V1ConnectionKind.WASB:
        from polyaxon.fs.azure import get_fs

        return get_fs(
            context_path=context_path,
            asynchronous=asynchronous,
            use_listings_cache=use_listings_cache,
            **kwargs
        )
    if connection_type.kind == V1ConnectionKind.S3:
        from polyaxon.fs.s3 import get_fs

        return get_fs(
            context_path=context_path,
            asynchronous=asynchronous,
            use_listings_cache=use_listings_cache,
            **kwargs
        )
    if connection_type.kind == V1ConnectionKind.GCS:
        from polyaxon.fs.gcs import get_fs

        return get_fs(
            context_path=context_path,
            asynchronous=asynchronous,
            use_listings_cache=use_listings_cache,
            **kwargs
        )


async def get_async_fs_from_type(connection_type: V1ConnectionType, **kwargs):
    fs = _get_fs_from_type(connection_type=connection_type, asynchronous=True, **kwargs)
    if fs.async_impl and hasattr(fs, "set_session"):
        await fs.set_session()
    return fs


def get_sync_fs_from_type(connection_type: V1ConnectionType, **kwargs):
    return _get_fs_from_type(connection_type=connection_type, **kwargs)


def get_fs_from_name(connection_name: str, asynchronous: bool = False, **kwargs):
    connection_type = get_connection_type(connection_name)
    return _get_fs_from_type(
        connection_type=connection_type, asynchronous=asynchronous, **kwargs
    )


async def get_default_fs(**kwargs):
    connection_type = get_artifacts_connection_type()
    return await get_async_fs_from_type(
        connection_type=connection_type, auto_mkdir=True, **kwargs
    )


async def close_fs(fs):
    if hasattr(fs, "close_session"):
        fs.close_session(fs.loop, fs.session)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging

from requests import HTTPError
from typing import Dict, List, Union

import polyaxon_sdk

from polyaxon.client.client import PolyaxonClient
from polyaxon.client.decorators import client_handler
from polyaxon.constants.globals import DEFAULT
from polyaxon.exceptions import PolyaxonClientException
from polyaxon.lifecycle import V1ProjectVersionKind, V1StageCondition
from polyaxon.utils.fqn_utils import get_entity_full_name, get_entity_info
from polyaxon.utils.query_params import get_query_params
from polyaxon.utils.validation import validate_tags
from polyaxon_sdk.rest import ApiException


class ProjectClient:
    """ProjectClient is a client to communicate with Polyaxon projects endpoints.

    If no values are passed to this class,
    Polyaxon will try to resolve the owner and project from the environment:
     * If you have a configured CLI, Polyaxon will use the configuration of the cli.
     * If you have a cached project using the CLI,
       the client will default to that cached project unless you override the values.
     * If you use this client in the context of a job or a service managed by Polyaxon,
       a configuration will be available to resolve the values based on that run.

    If you intend to create a new project instance or to list projects,
    only the `owner` parameter is required.

    Properties:
        project: str.
        owner: str.
        project_data: V1Project.

    Args:
        owner: str, optional, the owner is the username or
             the organization name owning this project.
        project: str, optional, project name.
        client: [PolyaxonClient](/docs/core/python-library/polyaxon-client/), optional,
             an instance of a configured client, if not passed,
             a new instance will be created based on the available environment.

    Raises:
        PolyaxonClientException: If no owner is passed and Polyaxon cannot
            resolve an owner from the environment.
    """

    @client_handler(check_no_op=True)
    def __init__(
        self,
        owner: str = None,
        project: str = None,
        client: PolyaxonClient = None,
    ):
        if not owner and project:
            owner, project = get_entity_info(
                get_entity_full_name(owner=owner, entity=project)
            )

        if not owner:
            raise PolyaxonClientException("Please provide a valid owner.")

        self._client = client
        self._owner = owner or DEFAULT
        self._project = project
        self._project_data = polyaxon_sdk.V1Project()

    @property
    def client(self):
        if self._client:
            return self._client
        self._client = PolyaxonClient()
        return self._client

    @property
    def owner(self):
        return self._owner

    @property
    def project(self):
        return self._project

    @property
    def project_data(self):
        return self._project_data

    @client_handler(check_no_op=True, check_offline=True)
    def refresh_data(self):
        """Fetches the project data from the api."""
        self._project_data = self.client.projects_v1.get_project(
            self.owner, self.project
        )
        if self._project_data.owner is None:
            self._project_data.owner = self.owner

    @client_handler(check_no_op=True, check_offline=True)
    def create(
        self, data: Union[Dict, polyaxon_sdk.V1Project]
    ) -> polyaxon_sdk.V1Project:
        """Creates a new project based on the data passed.

        [Project API](/docs/api/#operation/CreateProject)

        Args:
            data: dict or V1Project, required.

        Returns:
            V1Project, project instance from the response.
        """
        self._project_data = self.client.projects_v1.create_project(
            self.owner,
            data,
            async_req=False,
        )
        self._project_data.owner = self.owner
        self._project = self._project_data.name
        return self._project_data

    @client_handler(check_no_op=True, check_offline=True)
    def list(
        self, query: str = None, sort: str = None, limit: int = None, offset: int = None
    ) -> List[polyaxon_sdk.V1Project]:
        """Lists projects under the current owner.

        [Project API](/docs/api/#operation/ListProjects)

        Args:
            query: str, optional, query filters, please refer to
                    [Project PQL](/docs/core/query-language/projects/#query)
            sort: str, optional, fields to order by, please refer to
                    [Project PQL](/docs/core/query-language/projects/#sort)
            limit: int, optional, limit of projects to return.
            offset: int, optional, offset pages to paginate projects.

        Returns:
            List[V1Project], list of project instances.
        """
        params = get_query_params(limit=limit, offset=offset, query=query, sort=sort)
        return self.client.projects_v1.list_projects(self.owner, **params)

    @client_handler(check_no_op=True, check_offline=True)
    def delete(self):
        """Deletes project based on the current owner and project."""
        return self.client.projects_v1.delete_project(self.owner, self.project)

    @client_handler(check_no_op=True, check_offline=True)
    def update(
        self, data: Union[Dict, polyaxon_sdk.V1Project]
    ) -> polyaxon_sdk.V1Project:
        """Updates a project based on the data passed.

        [Project API](/docs/api/#operation/PatchProject)

        Args:
            data: Dict or V1Project, required.

        Returns:
            V1Project, project instance from the response.
        """
        self._project_data = self.client.projects_v1.patch_project(
            self.owner,
            self.project,
            body=data,
            async_req=False,
        )
        self._project = self._project_data.name
        return self._project_data

    @client_handler(check_no_op=True, check_offline=True)
    def list_runs(
        self, query: str = None, sort: str = None, limit: int = None, offset: int = None
    ):
        """Lists runs under the current owner/project.

        [Run API](/docs/api/#operation/ListRuns)

        Args:
            query: str, optional, query filters, please refer to
                 [Run PQL](/docs/core/query-language/runs/#query)
            sort: str, optional, fields to order by, please refer to
                 [Run PQL](/docs/core/query-language/runs/#sort)
            limit: int, optional, limit of runs to return.
            offset: int, optional, offset pages to paginate runs.

        Returns:
            List[V1Run], list of run instances.
        """
        params = get_query_params(
            limit=limit or 20, offset=offset, query=query, sort=sort
        )
        return self.client.runs_v1.list_runs(self.owner, self.project, **params)

    @client_handler(check_no_op=True, check_offline=True)
    def list_versions(
        self,
        kind: V1ProjectVersionKind,
        query: str = None,
        sort: str = None,
        limit: int = None,
        offset: int = None,
    ) -> List[polyaxon_sdk.V1ProjectVersion]:
        """Lists project versions under the current owner/project.

        [Project API](/docs/api/#operation/ListProjectVersions)

        Args:
            kind: V1ProjectVersionKind, kind of the project version.
            query: str, optional, query filters, please refer to
                 [Run PQL](/docs/core/query-language/runs/#query)
            sort: str, optional, fields to order by, please refer to
                 [Run PQL](/docs/core/query-language/runs/#sort)
            limit: int, optional, limit of runs to return.
            offset: int, optional, offset pages to paginate runs.

        Returns:
            List[V1ProjectVersion], list of versions.
        """
        params = get_query_params(
            limit=limit or 20, offset=offset, query=query, sort=sort
        )
        return self.client.projects_v1.list_versions(
            self.owner, self.project, kind, **params
        )

    @client_handler(check_no_op=True, check_offline=True)
    def get_version(
        self, kind: V1ProjectVersionKind, version: str
    ) -> polyaxon_sdk.V1ProjectVersion:
        """Get a project version under the current owner/project.

        [Project API](/docs/api/#operation/GetVersion)

        Args:
            kind: V1ProjectVersionKind, kind of the project version.
            version: str, required, the version name/tag.

        Returns:
            V1ProjectVersion.
        """
        response = self.client.projects_v1.get_version(
            self.owner, self.project, kind, version
        )
        if response.kind != kind:
            raise PolyaxonClientException("This version is not of kind `%s`." % kind)
        return response

    @client_handler(check_no_op=True, check_offline=True)
    def create_version(
        self,
        kind: V1ProjectVersionKind,
        data: Union[Dict, polyaxon_sdk.V1ProjectVersion],
    ) -> polyaxon_sdk.V1ProjectVersion:
        """Create a project version based on the data passed.

        [Project API](/docs/api/#operation/CreateVersion)

        Args:
            kind: V1ProjectVersionKind, kind of the project version.
            data: Dict or V1ProjectVersion, required.

        Returns:
            V1ProjectVersion.
        """
        if isinstance(data, polyaxon_sdk.V1ProjectVersion):
            data.kind = kind
        elif isinstance(data, dict):
            data["kind"] = kind
        return self.client.projects_v1.create_version(
            self.owner,
            self.project,
            kind,
            body=data,
            async_req=False,
        )

    @client_handler(check_no_op=True, check_offline=True)
    def patch_version(
        self,
        kind: V1ProjectVersionKind,
        version: str,
        data: Union[Dict, polyaxon_sdk.V1ProjectVersion],
    ) -> polyaxon_sdk.V1ProjectVersion:
        """Update a project version based on the data passed.

        [Project API](/docs/api/#operation/PatchVersion)

        Args:
            kind: V1ProjectVersionKind, kind of the project version.
            version: str, required, the version name/tag.
            data: Dict or V1ProjectVersion, required.

        Returns:
            V1ProjectVersion.
        """
        return self.client.projects_v1.patch_version(
            self.owner,
            self.project,
            kind,
            version,
            body=data,
            async_req=False,
        )

    @client_handler(check_no_op=True, check_offline=True)
    def register_version(
        self,
        kind: V1ProjectVersionKind,
        version: str,
        description: str = None,
        tags: Union[str, List[str]] = None,
        content: str = None,
        run: str = None,
        connection: str = None,
        artifacts: List[str] = None,
        force: bool = False,
    ) -> polyaxon_sdk.V1ProjectVersion:
        """Create or Update a project version based on the data passed.

        [Project API](/docs/api/#operation/PatchVersion)

        Args:
            kind: V1ProjectVersionKind, kind of the project version.
            version: str, optional, the version name/tag.
            description: str, optional, the version description.
            tags: str or List[str], optional.
            content: str, optional, content/metadata (JSON object) of the version.
            run: str, optional, a uuid reference to the run.
            connection: str, optional, a uuid reference to a connection.
            artifacts: List[str], optional, list of artifacts to highlight(requires passing a run)
            force: bool, optional, to force push, i.e. update if exists.

        Returns:
            V1ProjectVersion.
        """
        try:
            self.get_version(kind, version)
            if not force:
                raise PolyaxonClientException(
                    "A {} version {} already exists. "
                    "Please pass the force (--force for CLI) flag "
                    "if you want to push force this version.".format(kind, version)
                )
            to_update = True
        except (ApiException, HTTPError, AttributeError):
            to_update = False

        if tags is not None:
            tags = validate_tags(tags, validate_yaml=True)
        if artifacts is not None:
            artifacts = validate_tags(artifacts, validate_yaml=True)

        if to_update:
            version_config = polyaxon_sdk.V1ProjectVersion()
            if description is not None:
                version_config.description = description
            if tags:
                version_config.tags = tags
            if content:
                version_config.content = content
            if run:
                version_config.run = run
            if artifacts is not None:
                version_config.artifacts = artifacts
            if connection is not None:
                version_config.connection = connection
            return self.patch_version(
                kind=kind,
                version=version,
                data=version_config,
            )
        else:
            version_config = polyaxon_sdk.V1ProjectVersion(
                name=version,
                description=description,
                tags=tags,
                run=run,
                artifacts=artifacts,
                connection=connection,
                content=content,
            )
            return self.create_version(kind=kind, data=version_config)

    @client_handler(check_no_op=True, check_offline=True)
    def delete_version(self, kind: V1ProjectVersionKind, version: str):
        """Delete a project version under the current owner/project.

        [Project API](/docs/api/#operation/DeleteVersion)

        Args:
            kind: V1ProjectVersionKind, kind of the project version.
            version: str, required, the version name/tag.
        """
        logging.info("Deleting {} version: `{}`".format(kind, version))
        return self.client.projects_v1.delete_version(
            self.owner,
            self.project,
            kind,
            version,
            async_req=False,
        )

    @client_handler(check_no_op=True, check_offline=True)
    def stage_version(
        self,
        kind: V1ProjectVersionKind,
        version: str,
        condition: Union[Dict, V1StageCondition],
    ):
        """Create a new a project version stage.

        [Project API](/docs/api/#operation/CreateVersionStage)

        Args:
            kind: V1ProjectVersionKind, kind of the project version.
            version: str, required, the version name/tag.
            condition: Dict or V1StageCondition.
        """
        return self.client.projects_v1.create_version_stage(
            self.owner,
            self.project,
            kind,
            version,
            body={"condition": condition},
            async_req=False,
        )

    @client_handler(check_no_op=True, check_offline=True)
    def transfer_version(
        self, kind: V1ProjectVersionKind, version: str, to_project: str
    ):
        """Transfers the version to a project under the same owner/organization.

        [Run API](/docs/api/#operation/TransferRun)

        Args:
            kind: V1ProjectVersionKind, kind of the project version.
            version: str, required, the version name/tag.
            to_project: str, required, the destination project to transfer the version to.
        """

        return self.client.projects_v1.transfer_version(
            self.owner,
            self.project,
            kind,
            version,
            body={"project": to_project},
            async_req=False,
        )

    @client_handler(check_no_op=True, check_offline=True)
    def copy_version(
        self,
        kind: V1ProjectVersionKind,
        version: str,
        to_project: str = None,
        name: str = None,
        description: str = None,
        tags: Union[str, List[str]] = None,
        content: str = None,
        force: bool = False,
    ) -> polyaxon_sdk.V1ProjectVersion:
        """Copies the version to the same project or to a destination project.

        If `to_project` is provided,
        the version will be copied to the destination project under the same owner/organization.

        If `name` is provided the version will be copied with the new name,
        otherwise the copied version will be have a suffix `-copy`.


        [Run API](/docs/api/#operation/TransferRun)

        Args:
            kind: V1ProjectVersionKind, kind of the project version.
            version: str, required, the version name/tag.
            to_project: str, optional, the destination project to copy the version to.
            name: str, optional, the name to use for registering the copied version,
                 default value is the original version's name with `-copy` prefix.
            description: str, optional, the version description,
                 default value is the original version's description.
            tags: str or List[str], optional, the version description,
                 default value is the original version's description.
            content: str, optional, content/metadata (JSON object) of the version,
                 default value is the original version's content.
            force: bool, optional, to force push, i.e. update if exists.
        """
        original_version = self.get_version(kind, version)
        version = name if name else "{}-copy".format(version)
        return ProjectClient(
            owner=self.owner,
            project=to_project or self.project,
            client=self.client,
        ).register_version(
            kind=kind,
            version=version,
            description=description or original_version.description,
            tags=tags or original_version.tags,
            content=content or original_version.content,
            run=original_version.run,
            connection=original_version.connection,
            artifacts=original_version.artifacts,
            force=force,
        )



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from typing import Any, Dict, Iterable, List, Optional

from polyaxon import pkg, settings
from polyaxon.api import VERSION_V1
from polyaxon.auxiliaries import V1PolyaxonInitContainer, V1PolyaxonSidecarContainer
from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.containers.names import INIT_PREFIX, SIDECAR_PREFIX
from polyaxon.env_vars.keys import POLYAXON_KEYS_NO_API
from polyaxon.exceptions import PolypodException
from polyaxon.k8s import k8s_schemas
from polyaxon.polyflow import V1Environment, V1Init, V1Plugins
from polyaxon.polypod.common.containers import ensure_container_name, sanitize_container
from polyaxon.polypod.common.env_vars import (
    get_base_env_vars,
    get_env_var,
    get_proxy_env_vars,
    get_service_env_vars,
)
from polyaxon.polypod.common.mounts import get_mounts
from polyaxon.polypod.init.artifacts import get_artifacts_path_container
from polyaxon.polypod.init.auth import get_auth_context_container
from polyaxon.polypod.init.custom import get_custom_init_container
from polyaxon.polypod.init.dockerfile import get_dockerfile_init_container
from polyaxon.polypod.init.file import get_file_init_container
from polyaxon.polypod.init.git import get_git_init_container
from polyaxon.polypod.init.store import get_store_container
from polyaxon.polypod.main.container import get_main_container
from polyaxon.polypod.pod.volumes import get_pod_volumes
from polyaxon.polypod.sidecar.container import get_sidecar_container
from polyaxon.polypod.specs.contexts import PluginsContextsSpec
from polyaxon.polypod.specs.replica import ReplicaSpec
from polyaxon.schemas.types import V1ConnectionType, V1K8sResourceType
from polyaxon.services.auth import AuthenticationTypes
from polyaxon.services.headers import PolyaxonServiceHeaders, PolyaxonServices
from polyaxon.utils.fqn_utils import get_resource_name, get_run_instance
from polyaxon.utils.host_utils import get_api_host
from polyaxon.utils.http_utils import clean_host
from polyaxon.utils.list_utils import to_list
from polyaxon.utils.sanitizers import sanitize_string_dict
from polyaxon.utils.string_utils import slugify


class ConverterAbstract:
    def get_main_env_vars(
        self, external_host: bool = False, **kwargs
    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
        raise NotImplementedError

    def get_polyaxon_sidecar_service_env_vars(
        self,
        external_host: bool = False,
    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
        raise NotImplementedError

    def get_auth_service_env_vars(
        self, external_host: bool = False
    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
        raise NotImplementedError

    def get_init_service_env_vars(
        self, external_host: bool = False
    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
        raise NotImplementedError


class BaseConverter(ConverterAbstract):
    SPEC_KIND = None
    GROUP = None
    API_VERSION = None
    PLURAL = None
    K8S_ANNOTATIONS_KIND = None
    K8S_LABELS_COMPONENT = None
    K8S_LABELS_PART_OF = None
    MAIN_CONTAINER_ID = None

    def __init__(
        self,
        owner_name: str,
        project_name: str,
        run_name: str,
        run_uuid: str,
        run_path: Optional[str] = None,
        namespace: str = "default",
        internal_auth: bool = False,
        polyaxon_sidecar: V1PolyaxonSidecarContainer = None,
        polyaxon_init: V1PolyaxonInitContainer = None,
    ):
        self.is_valid()
        self.owner_name = owner_name
        self.project_name = project_name
        self.run_name = run_name
        self.run_uuid = run_uuid
        self.run_path = run_path or self.run_uuid
        self.resource_name = self.get_resource_name()
        self.run_instance = self.get_instance()
        self.namespace = namespace
        self.internal_auth = internal_auth
        self.polyaxon_sidecar = polyaxon_sidecar
        self.polyaxon_init = polyaxon_init

    def get_instance(self):
        return get_run_instance(
            owner=self.owner_name, project=self.project_name, run_uuid=self.run_uuid
        )

    def get_resource_name(self):
        return get_resource_name(self.run_uuid)

    def is_valid(self):
        if not self.SPEC_KIND:
            raise PolypodException(
                "Please make sure that a spawner subclass has a valid SPEC_KIND"
            )
        if not self.GROUP:
            raise PolypodException(
                "Please make sure that a spawner subclass has a valid GROUP"
            )
        if not self.API_VERSION:
            raise PolypodException(
                "Please make sure that a spawner subclass has a valid API_VERSION"
            )
        if not self.PLURAL:
            raise PolypodException(
                "Please make sure that a spawner subclass has a valid PLURAL"
            )
        if not self.K8S_ANNOTATIONS_KIND:
            raise PolypodException(
                "Please make sure that a spawner subclass has a valid K8S_ANNOTATIONS_KIND"
            )
        if not self.K8S_LABELS_COMPONENT:
            raise PolypodException(
                "Please make sure that a spawner subclass has a valid K8S_LABELS_COMPONENT"
            )
        if not self.K8S_LABELS_PART_OF:
            raise PolypodException(
                "Please make sure that a spawner subclass has a valid K8S_LABELS_PART_OF"
            )

    def get_recommended_labels(self, version: str):
        return {
            "app.kubernetes.io/name": slugify(
                self.run_name[:63] if self.run_name else self.run_name
            ),
            "app.kubernetes.io/instance": self.run_uuid,
            "app.kubernetes.io/version": version,
            "app.kubernetes.io/part-of": self.K8S_LABELS_PART_OF,
            "app.kubernetes.io/component": self.K8S_LABELS_COMPONENT,
            "app.kubernetes.io/managed-by": "polyaxon",
        }

    @property
    def annotations(self):
        return {
            "operation.polyaxon.com/name": self.run_name,
            "operation.polyaxon.com/owner": self.owner_name,
            "operation.polyaxon.com/project": self.project_name,
            "operation.polyaxon.com/kind": self.K8S_ANNOTATIONS_KIND,
        }

    def get_annotations(self, annotations: Dict):
        annotations = annotations or {}
        annotations = copy.copy(annotations)
        annotations.update(self.annotations)
        return sanitize_string_dict(annotations)

    def get_labels(self, version: str, labels: Dict):
        labels = labels or {}
        labels = copy.copy(labels)
        labels.update(self.get_recommended_labels(version=version))
        return sanitize_string_dict(labels)

    @staticmethod
    def get_by_name(values: List[Any]):
        return {c.name: c for c in values}

    def get_main_env_vars(
        self, external_host: bool = False, **kwargs
    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
        return get_base_env_vars(settings.AGENT_CONFIG.use_proxy_env_vars_use_in_ops)

    def get_polyaxon_sidecar_service_env_vars(
        self,
        external_host: bool = False,
    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
        env = []
        if settings.CLIENT_CONFIG.no_api:
            env += [get_env_var(name=POLYAXON_KEYS_NO_API, value=True)]
        proxy_env = get_proxy_env_vars(
            settings.AGENT_CONFIG.use_proxy_env_vars_use_in_ops
        )
        if proxy_env:
            env += proxy_env
        return env

    def get_auth_service_env_vars(
        self, external_host: bool = False
    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
        return None

    def get_init_service_env_vars(
        self, external_host: bool = False
    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
        env = []
        if settings.CLIENT_CONFIG.no_api:
            env += [get_env_var(name=POLYAXON_KEYS_NO_API, value=True)]
        proxy_env = get_proxy_env_vars(
            settings.AGENT_CONFIG.use_proxy_env_vars_use_in_ops
        )
        if proxy_env:
            env += proxy_env
        return env

    def get_main_container(
        self,
        main_container: k8s_schemas.V1Container,
        contexts: PluginsContextsSpec,
        artifacts_store: V1ConnectionType,
        connections: List[str],
        init_connections: Optional[List[V1Init]],
        connection_by_names: Dict[str, V1ConnectionType],
        log_level: str,
        secrets: Optional[Iterable[V1K8sResourceType]],
        config_maps: Optional[Iterable[V1K8sResourceType]],
        kv_env_vars: List[List] = None,
        ports: List[int] = None,
    ) -> k8s_schemas.V1Container:
        env = self.get_main_env_vars(
            external_host=contexts.external_host if contexts else False
        )
        volume_mounts = get_mounts(
            use_auth_context=contexts.auth,
            use_artifacts_context=False,  # Main container has a check and handling for this
            use_docker_context=contexts.docker,
            use_shm_context=contexts.shm,
        )

        return get_main_container(
            container_id=self.MAIN_CONTAINER_ID,
            main_container=main_container,
            volume_mounts=volume_mounts,
            log_level=log_level,
            contexts=contexts,
            artifacts_store=artifacts_store,
            connections=connections,
            init=init_connections,
            connection_by_names=connection_by_names,
            secrets=secrets,
            config_maps=config_maps,
            kv_env_vars=kv_env_vars,
            env=env,
            ports=ports,
            run_path=self.run_path,
        )

    def get_sidecar_containers(
        self,
        polyaxon_sidecar: V1PolyaxonSidecarContainer,
        contexts: PluginsContextsSpec,
        artifacts_store: V1ConnectionType,
        sidecar_containers: List[k8s_schemas.V1Container],
        log_level: str = None,
    ) -> List[k8s_schemas.V1Container]:
        sidecar_containers = [
            ensure_container_name(container=c, prefix=SIDECAR_PREFIX)
            for c in to_list(sidecar_containers, check_none=True)
        ]
        polyaxon_sidecar_container = get_sidecar_container(
            container_id=self.MAIN_CONTAINER_ID,
            polyaxon_sidecar=polyaxon_sidecar,
            env=self.get_polyaxon_sidecar_service_env_vars(
                external_host=contexts.external_host if contexts else False
            ),
            artifacts_store=artifacts_store,
            contexts=contexts,
            run_path=self.run_path,
            log_level=log_level,
        )
        containers = to_list(polyaxon_sidecar_container, check_none=True)
        containers += sidecar_containers
        return [sanitize_container(c) for c in containers]

    def handle_init_connections(
        self,
        polyaxon_init: V1PolyaxonInitContainer,
        artifacts_store: V1ConnectionType,
        init_connections: List[V1Init],
        connection_by_names: Dict[str, V1ConnectionType],
        contexts: PluginsContextsSpec,
    ) -> List[k8s_schemas.V1Container]:
        containers = []
        external_host = contexts.external_host if contexts else False

        # Prepare connections that Polyaxon can init automatically
        for init_connection in init_connections:
            if init_connection.connection:
                connection_spec = connection_by_names.get(init_connection.connection)
                # Handling ssh with git
                if (
                    V1ConnectionKind.is_ssh(connection_spec.kind)
                    and init_connection.git
                ):
                    connection_spec.schema.patch_git(init_connection.git)
                    containers.append(
                        get_git_init_container(
                            polyaxon_init=polyaxon_init,
                            connection=connection_spec,
                            container=init_connection.container,
                            env=self.get_init_service_env_vars(
                                external_host=external_host
                            ),
                            mount_path=init_connection.path,
                            contexts=contexts,
                            track=True,
                        )
                    )
                elif V1ConnectionKind.is_git(connection_spec.kind):
                    if init_connection.git:  # Update the default schema
                        connection_spec.schema.patch(init_connection.git)
                    containers.append(
                        get_git_init_container(
                            polyaxon_init=polyaxon_init,
                            connection=connection_spec,
                            container=init_connection.container,
                            env=self.get_init_service_env_vars(
                                external_host=external_host
                            ),
                            mount_path=init_connection.path,
                            contexts=contexts,
                            track=True,
                        )
                    )
                elif V1ConnectionKind.is_artifact(connection_spec.kind):
                    containers.append(
                        get_store_container(
                            polyaxon_init=polyaxon_init,
                            connection=connection_spec,
                            artifacts=init_connection.artifacts,
                            container=init_connection.container,
                            env=self.get_init_service_env_vars(
                                external_host=external_host
                            ),
                            mount_path=init_connection.path,
                            is_default_artifacts_store=artifacts_store
                            and init_connection.connection == artifacts_store.name,
                        )
                    )
                else:
                    containers.append(
                        get_custom_init_container(
                            connection=connection_spec,
                            container=init_connection.container,
                            env=self.get_init_service_env_vars(
                                external_host=external_host
                            ),
                            mount_path=init_connection.path,
                            contexts=contexts,
                        )
                    )
            else:
                # artifacts init without connection should default to the artifactsStore
                if init_connection.artifacts:
                    containers.append(
                        get_store_container(
                            polyaxon_init=polyaxon_init,
                            connection=artifacts_store,
                            artifacts=init_connection.artifacts,
                            container=init_connection.container,
                            env=self.get_init_service_env_vars(
                                external_host=external_host
                            ),
                            mount_path=init_connection.path,
                            is_default_artifacts_store=True,
                        )
                    )
                # git init without connection
                if init_connection.git:
                    git_name = init_connection.git.get_name()
                    containers.append(
                        get_git_init_container(
                            polyaxon_init=polyaxon_init,
                            connection=V1ConnectionType(
                                name=git_name,
                                kind=V1ConnectionKind.GIT,
                                schema=init_connection.git,
                                secret=None,
                            ),
                            container=init_connection.container,
                            env=self.get_init_service_env_vars(
                                external_host=external_host
                            ),
                            mount_path=init_connection.path,
                            contexts=contexts,
                            track=False,
                        )
                    )
                # Dockerfile initialization
                if init_connection.dockerfile:
                    containers.append(
                        get_dockerfile_init_container(
                            polyaxon_init=polyaxon_init,
                            dockerfile_args=init_connection.dockerfile,
                            env=self.get_init_service_env_vars(
                                external_host=external_host
                            ),
                            mount_path=init_connection.path,
                            contexts=contexts,
                            run_path=self.run_path,
                            run_instance=self.run_instance,
                        )
                    )
                # File initialization
                if init_connection.file:
                    containers.append(
                        get_file_init_container(
                            polyaxon_init=polyaxon_init,
                            file_args=init_connection.file,
                            env=self.get_init_service_env_vars(
                                external_host=external_host
                            ),
                            mount_path=init_connection.path,
                            contexts=contexts,
                            run_path=self.run_path,
                            run_instance=self.run_instance,
                        )
                    )

        return containers

    def get_init_containers(
        self,
        polyaxon_init: V1PolyaxonInitContainer,
        contexts: PluginsContextsSpec,
        artifacts_store: V1ConnectionType,
        init_connections: List[V1Init],
        init_containers: List[k8s_schemas.V1Container],
        connection_by_names: Dict[str, V1ConnectionType],
    ) -> List[k8s_schemas.V1Container]:
        init_containers = [
            ensure_container_name(container=c, prefix=INIT_PREFIX)
            for c in to_list(init_containers, check_none=True)
        ]
        init_connections = to_list(init_connections, check_none=True)
        containers = []

        # Add auth context
        if contexts and contexts.auth:
            containers.append(
                get_auth_context_container(
                    polyaxon_init=polyaxon_init,
                    env=self.get_auth_service_env_vars(
                        external_host=contexts.external_host
                    ),
                )
            )

        # Add outputs
        if contexts and contexts.collect_artifacts:
            containers += to_list(
                get_artifacts_path_container(
                    polyaxon_init=polyaxon_init,
                    artifacts_store=artifacts_store,
                    run_path=self.run_path,
                    auto_resume=contexts.auto_resume,
                    env=get_proxy_env_vars(
                        settings.AGENT_CONFIG.use_proxy_env_vars_use_in_ops
                    ),
                ),
                check_none=True,
            )

        containers += self.handle_init_connections(
            polyaxon_init=polyaxon_init,
            artifacts_store=artifacts_store,
            init_connections=init_connections,
            connection_by_names=connection_by_names,
            contexts=contexts,
        )
        init_containers = containers + init_containers
        return [sanitize_container(c) for c in init_containers]

    def filter_containers_from_init(
        self, init: List[V1Init]
    ) -> List[k8s_schemas.V1Container]:
        return [i.container for i in init if not i.has_connection()]

    def filter_connections_from_init(self, init: List[V1Init]) -> List[V1Init]:
        return [i for i in init if i.has_connection()]

    def get_replica_resource(
        self,
        environment: V1Environment,
        plugins: V1Plugins,
        contexts: PluginsContextsSpec,
        volumes: List[k8s_schemas.V1Volume],
        init: List[V1Init],
        sidecars: List[k8s_schemas.V1Container],
        container: k8s_schemas.V1Container,
        artifacts_store: V1ConnectionType,
        connections: List[str],
        connection_by_names: Dict[str, V1ConnectionType],
        secrets: Optional[Iterable[V1K8sResourceType]],
        config_maps: Optional[Iterable[V1K8sResourceType]],
        kv_env_vars: List[List],
        default_sa: str = None,
        ports: List[int] = None,
        num_replicas: int = None,
    ) -> ReplicaSpec:
        volumes = volumes or []
        init = init or []
        sidecars = sidecars or []
        connections = connections or []
        environment = environment or V1Environment()
        environment.service_account_name = (
            environment.service_account_name
            or default_sa
            or settings.AGENT_CONFIG.runs_sa
        )

        init_connections = self.filter_connections_from_init(init=init)

        volumes = get_pod_volumes(
            contexts=contexts,
            artifacts_store=artifacts_store,
            init_connections=init_connections,
            connections=connections,
            connection_by_names=connection_by_names,
            secrets=secrets,
            config_maps=config_maps,
            volumes=volumes,
        )

        init_containers = self.get_init_containers(
            polyaxon_init=self.polyaxon_init,
            contexts=contexts,
            artifacts_store=artifacts_store,
            init_connections=init_connections,
            init_containers=self.filter_containers_from_init(init=init),
            connection_by_names=connection_by_names,
        )

        sidecar_containers = self.get_sidecar_containers(
            polyaxon_sidecar=self.polyaxon_sidecar,
            contexts=contexts,
            artifacts_store=artifacts_store,
            sidecar_containers=sidecars,
        )

        main_container = self.get_main_container(
            main_container=container,
            contexts=contexts,
            artifacts_store=artifacts_store,
            connections=connections,
            init_connections=init_connections,
            connection_by_names=connection_by_names,
            log_level=plugins.log_level,
            secrets=secrets,
            config_maps=config_maps,
            ports=ports,
            kv_env_vars=kv_env_vars,
        )

        labels = self.get_labels(version=pkg.VERSION, labels=environment.labels)
        annotations = self.get_annotations(annotations=environment.annotations)
        return ReplicaSpec(
            volumes=volumes,
            init_containers=init_containers,
            sidecar_containers=sidecar_containers,
            main_container=main_container,
            labels=labels,
            annotations=annotations,
            environment=environment,
            num_replicas=num_replicas,
        )

    def get_resource(self, **kwargs) -> Dict:
        raise NotImplementedError


class PlatformConverterMixin(ConverterAbstract):
    def get_api_host(self, external_host: bool = False):
        if external_host:
            return get_api_host(default=settings.CLIENT_CONFIG.host)
        else:
            return clean_host(settings.CLIENT_CONFIG.host)

    def get_service_env_vars(
        self,
        service_header: str,
        header: str = None,
        include_secret_key: bool = False,
        include_internal_token: bool = False,
        include_agent_token: bool = False,
        authentication_type: str = None,
        external_host: bool = False,
    ) -> List[k8s_schemas.V1EnvVar]:
        header = header or PolyaxonServiceHeaders.SERVICE
        return get_service_env_vars(
            header=header,
            service_header=service_header,
            authentication_type=authentication_type,
            include_secret_key=include_secret_key,
            include_internal_token=include_internal_token,
            include_agent_token=include_agent_token,
            polyaxon_default_secret_ref=settings.AGENT_CONFIG.app_secret_name,
            polyaxon_agent_secret_ref=settings.AGENT_CONFIG.agent_secret_name,
            api_host=self.get_api_host(external_host),
            api_version=VERSION_V1,
            run_instance=self.run_instance,
            use_proxy_env_vars_use_in_ops=settings.AGENT_CONFIG.use_proxy_env_vars_use_in_ops,
        )

    def get_main_env_vars(self, external_host: bool = False, **kwargs):
        return self.get_service_env_vars(
            service_header=PolyaxonServices.RUNNER,
            external_host=external_host,
        )

    def get_auth_service_env_vars(
        self, external_host: bool = False
    ) -> List[k8s_schemas.V1EnvVar]:
        return self.get_service_env_vars(
            service_header=PolyaxonServices.INITIALIZER,
            include_internal_token=self.internal_auth,
            include_agent_token=not self.internal_auth,
            authentication_type=(
                AuthenticationTypes.INTERNAL_TOKEN
                if self.internal_auth
                else AuthenticationTypes.TOKEN
            ),
            header=(
                PolyaxonServiceHeaders.INTERNAL
                if self.internal_auth
                else PolyaxonServiceHeaders.SERVICE
            ),
            external_host=external_host,
        )

    def get_polyaxon_sidecar_service_env_vars(
        self, external_host: bool = False
    ) -> List[k8s_schemas.V1EnvVar]:
        return self.get_service_env_vars(
            service_header=PolyaxonServices.SIDECAR,
            authentication_type=AuthenticationTypes.TOKEN,
            header=PolyaxonServiceHeaders.SERVICE,
            external_host=external_host,
        )

    def get_init_service_env_vars(
        self, external_host: bool = False
    ) -> List[k8s_schemas.V1EnvVar]:
        return self.get_service_env_vars(
            service_header=PolyaxonServices.INITIALIZER,
            authentication_type=AuthenticationTypes.TOKEN,
            header=PolyaxonServiceHeaders.SERVICE,
            external_host=external_host,
        )



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from polyaxon.connections.base import BaseService


class AzureService(BaseService):
    def __init__(self, connection=None, **kwargs):
        super().__init__(connection=connection, **kwargs)
        self._account_name = kwargs.get("account_name") or kwargs.get(
            "AZURE_ACCOUNT_NAME"
        )
        self._account_key = kwargs.get("account_key") or kwargs.get("AZURE_ACCOUNT_KEY")
        self._connection_string = kwargs.get("connection_string") or kwargs.get(
            "AZURE_CONNECTION_STRING"
        )

    def set_connection(
        self,
        connection=None,
        connection_name=None,
        account_name=None,
        account_key=None,
        connection_string=None,
    ):
        raise NotImplementedError

    def set_env_vars(self):
        if self._account_name:
            os.environ["AZURE_ACCOUNT_NAME"] = self._account_name
        if self._account_key:
            os.environ["AZURE_ACCOUNT_KEY"] = self._account_key
        if self._connection_string:
            os.environ["AZURE_CONNECTION_STRING"] = self._connection_string



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import uuid

from marshmallow import fields, validate

import polyaxon_sdk

from polyaxon.polyboard.artifacts.kinds import V1ArtifactKind
from polyaxon.schemas.base import BaseConfig, BaseSchema


class RunArtifactSchema(BaseSchema):
    name = fields.Str(allow_none=True)
    kind = fields.Str(
        allow_none=True, validate=validate.OneOf(V1ArtifactKind.allowable_values)
    )
    path = fields.Str(allow_none=True)
    state = fields.Str(allow_none=True)
    summary = fields.Dict(allow_none=True)
    connection = fields.Str(allow_none=True)
    is_input = fields.Bool(allow_none=True)

    @staticmethod
    def schema_config():
        return V1RunArtifact


class V1RunArtifact(BaseConfig, polyaxon_sdk.V1RunArtifact):
    IDENTIFIER = "artifact"
    SCHEMA = RunArtifactSchema
    REDUCED_ATTRIBUTES = [
        "name",
        "kind",
        "path",
        "state",
        "summary",
        "connection",
        "is_input",
    ]

    @classmethod
    def from_model(cls, model):
        return cls(
            name=model.name,
            kind=model.kind,
            path=model.path,
            state=model.state,
            summary=model.summary,
            # connection=model.connection,  # TODO: enable
        )

    def get_state(self, namespace: uuid.UUID):
        if self.state:
            return self.state

        if self.path:
            return uuid.uuid5(namespace, self.path)

        return uuid.uuid5(namespace, str(self.summary))



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List, Optional

from polyaxon.auxiliaries import V1PolyaxonInitContainer
from polyaxon.containers.contexts import (
    CONTEXT_MOUNT_ARTIFACTS,
    CONTEXT_MOUNT_RUN_OUTPUTS_FORMAT,
)
from polyaxon.containers.names import (
    INIT_FILE_CONTAINER_PREFIX,
    generate_container_name,
)
from polyaxon.k8s import k8s_schemas
from polyaxon.polypod.common import constants
from polyaxon.polypod.common.containers import patch_container
from polyaxon.polypod.common.env_vars import get_run_instance_env_var
from polyaxon.polypod.common.mounts import (
    get_auth_context_mount,
    get_connections_context_mount,
)
from polyaxon.polypod.common.volumes import get_volume_name
from polyaxon.polypod.specs.contexts import PluginsContextsSpec
from polyaxon.schemas.types import V1FileType
from polyaxon.utils.list_utils import to_list


def get_file_init_container(
    polyaxon_init: V1PolyaxonInitContainer,
    file_args: V1FileType,
    contexts: PluginsContextsSpec,
    run_path: str,
    run_instance: str,
    env: List[k8s_schemas.V1EnvVar] = None,
    mount_path: Optional[str] = None,
) -> k8s_schemas.V1Container:
    env = to_list(env, check_none=True)
    env = env + [get_run_instance_env_var(run_instance)]

    volume_name = (
        get_volume_name(mount_path)
        if mount_path
        else constants.CONTEXT_VOLUME_ARTIFACTS
    )
    mount_path = mount_path or CONTEXT_MOUNT_ARTIFACTS
    volume_mounts = [
        get_connections_context_mount(name=volume_name, mount_path=mount_path)
    ]
    if contexts and contexts.auth:
        volume_mounts.append(get_auth_context_mount(read_only=True))

    file_args.filename = file_args.filename or "file"
    container = k8s_schemas.V1Container(
        name=generate_container_name(INIT_FILE_CONTAINER_PREFIX),
        image=polyaxon_init.get_image(),
        image_pull_policy=polyaxon_init.image_pull_policy,
        command=["polyaxon", "initializer", "file"],
        args=[
            "--file-context={}".format(file_args.to_dict(dump=True)),
            "--filepath={}".format(mount_path),
            "--copy-path={}".format(CONTEXT_MOUNT_RUN_OUTPUTS_FORMAT.format(run_path)),
            "--track",
        ],
        env=env,
        resources=polyaxon_init.get_resources(),
        volume_mounts=volume_mounts,
    )
    return patch_container(container)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, List, Optional, Tuple

from polyaxon.exceptions import PolypodException
from polyaxon.k8s import k8s_schemas
from polyaxon.polyflow import V1Environment
from polyaxon.utils.list_utils import to_list


def get_pod_spec(
    resource_name: str,
    namespace: str,
    main_container: k8s_schemas.V1Container,
    sidecar_containers: Optional[List[k8s_schemas.V1Container]],
    init_containers: Optional[List[k8s_schemas.V1Container]],
    environment: Optional[V1Environment],
    labels: Dict[str, str],
    annotations: Dict[str, str],
    volumes: Optional[List[k8s_schemas.V1Volume]],
) -> Tuple[k8s_schemas.V1ObjectMeta, k8s_schemas.V1PodSpec]:
    if not main_container:
        raise PolypodException("A main container is required")
    environment = environment or V1Environment()

    metadata = k8s_schemas.V1ObjectMeta(
        name=resource_name,
        namespace=namespace,
        labels=labels,
        annotations=annotations,
    )

    init_containers = to_list(init_containers, check_none=True)
    containers = [main_container] + to_list(sidecar_containers, check_none=True)
    image_pull_secrets = None
    if environment.image_pull_secrets:
        image_pull_secrets = [
            k8s_schemas.V1LocalObjectReference(name=i)
            for i in environment.image_pull_secrets
        ]

    pod_spec = k8s_schemas.V1PodSpec(
        init_containers=init_containers,
        containers=containers,
        volumes=volumes,
        restart_policy=environment.restart_policy,
        image_pull_secrets=image_pull_secrets,
        security_context=environment.security_context,
        service_account_name=environment.service_account_name,
        node_selector=environment.node_selector,
        tolerations=environment.tolerations,
        affinity=environment.affinity,
        dns_config=environment.dns_config,
        dns_policy=environment.dns_policy,
        host_aliases=environment.host_aliases,
        host_network=environment.host_network,
        host_pid=environment.host_pid,
        node_name=environment.node_name,
        priority=environment.priority,
        priority_class_name=environment.priority_class_name,
        scheduler_name=environment.scheduler_name,
    )
    return metadata, pod_spec


def get_pod_template_spec(
    metadata: k8s_schemas.V1ObjectMeta, pod_spec: k8s_schemas.V1PodSpec
) -> k8s_schemas.V1PodTemplateSpec:
    return k8s_schemas.V1PodTemplateSpec(metadata=metadata, spec=pod_spec)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    Polyaxon SDKs and REST API specification.

    Polyaxon SDKs and REST API specification.  # noqa: E501

    The version of the OpenAPI document: 1.16.0
    Contact: contact@polyaxon.com
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from polyaxon_sdk.configuration import Configuration


class V1Token(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'uuid': 'str',
        'key': 'str',
        'name': 'str',
        'scopes': 'list[str]',
        'services': 'list[str]',
        'started_at': 'datetime',
        'expires_at': 'datetime',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'expiration': 'int'
    }

    attribute_map = {
        'uuid': 'uuid',
        'key': 'key',
        'name': 'name',
        'scopes': 'scopes',
        'services': 'services',
        'started_at': 'started_at',
        'expires_at': 'expires_at',
        'created_at': 'created_at',
        'updated_at': 'updated_at',
        'expiration': 'expiration'
    }

    def __init__(self, uuid=None, key=None, name=None, scopes=None, services=None, started_at=None, expires_at=None, created_at=None, updated_at=None, expiration=None, local_vars_configuration=None):  # noqa: E501
        """V1Token - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._uuid = None
        self._key = None
        self._name = None
        self._scopes = None
        self._services = None
        self._started_at = None
        self._expires_at = None
        self._created_at = None
        self._updated_at = None
        self._expiration = None
        self.discriminator = None

        if uuid is not None:
            self.uuid = uuid
        if key is not None:
            self.key = key
        if name is not None:
            self.name = name
        if scopes is not None:
            self.scopes = scopes
        if services is not None:
            self.services = services
        if started_at is not None:
            self.started_at = started_at
        if expires_at is not None:
            self.expires_at = expires_at
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if expiration is not None:
            self.expiration = expiration

    @property
    def uuid(self):
        """Gets the uuid of this V1Token.  # noqa: E501


        :return: The uuid of this V1Token.  # noqa: E501
        :rtype: str
        """
        return self._uuid

    @uuid.setter
    def uuid(self, uuid):
        """Sets the uuid of this V1Token.


        :param uuid: The uuid of this V1Token.  # noqa: E501
        :type uuid: str
        """

        self._uuid = uuid

    @property
    def key(self):
        """Gets the key of this V1Token.  # noqa: E501


        :return: The key of this V1Token.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this V1Token.


        :param key: The key of this V1Token.  # noqa: E501
        :type key: str
        """

        self._key = key

    @property
    def name(self):
        """Gets the name of this V1Token.  # noqa: E501


        :return: The name of this V1Token.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this V1Token.


        :param name: The name of this V1Token.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def scopes(self):
        """Gets the scopes of this V1Token.  # noqa: E501


        :return: The scopes of this V1Token.  # noqa: E501
        :rtype: list[str]
        """
        return self._scopes

    @scopes.setter
    def scopes(self, scopes):
        """Sets the scopes of this V1Token.


        :param scopes: The scopes of this V1Token.  # noqa: E501
        :type scopes: list[str]
        """

        self._scopes = scopes

    @property
    def services(self):
        """Gets the services of this V1Token.  # noqa: E501


        :return: The services of this V1Token.  # noqa: E501
        :rtype: list[str]
        """
        return self._services

    @services.setter
    def services(self, services):
        """Sets the services of this V1Token.


        :param services: The services of this V1Token.  # noqa: E501
        :type services: list[str]
        """

        self._services = services

    @property
    def started_at(self):
        """Gets the started_at of this V1Token.  # noqa: E501


        :return: The started_at of this V1Token.  # noqa: E501
        :rtype: datetime
        """
        return self._started_at

    @started_at.setter
    def started_at(self, started_at):
        """Sets the started_at of this V1Token.


        :param started_at: The started_at of this V1Token.  # noqa: E501
        :type started_at: datetime
        """

        self._started_at = started_at

    @property
    def expires_at(self):
        """Gets the expires_at of this V1Token.  # noqa: E501


        :return: The expires_at of this V1Token.  # noqa: E501
        :rtype: datetime
        """
        return self._expires_at

    @expires_at.setter
    def expires_at(self, expires_at):
        """Sets the expires_at of this V1Token.


        :param expires_at: The expires_at of this V1Token.  # noqa: E501
        :type expires_at: datetime
        """

        self._expires_at = expires_at

    @property
    def created_at(self):
        """Gets the created_at of this V1Token.  # noqa: E501


        :return: The created_at of this V1Token.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this V1Token.


        :param created_at: The created_at of this V1Token.  # noqa: E501
        :type created_at: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this V1Token.  # noqa: E501


        :return: The updated_at of this V1Token.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this V1Token.


        :param updated_at: The updated_at of this V1Token.  # noqa: E501
        :type updated_at: datetime
        """

        self._updated_at = updated_at

    @property
    def expiration(self):
        """Gets the expiration of this V1Token.  # noqa: E501


        :return: The expiration of this V1Token.  # noqa: E501
        :rtype: int
        """
        return self._expiration

    @expiration.setter
    def expiration(self, expiration):
        """Sets the expiration of this V1Token.


        :param expiration: The expiration of this V1Token.  # noqa: E501
        :type expiration: int
        """

        self._expiration = expiration

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1Token):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1Token):
            return True

        return self.to_dict() != other.to_dict()



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.api import K8S_V1_LOCATION, STREAMS_V1_LOCATION
from polyaxon.proxies.schemas.base import get_config

AUTH_OPTIONS = r"""
    auth_request     {auth_api};
    auth_request_set $auth_status $upstream_status;
"""  # noqa


def get_auth_config():
    return get_config(
        options=AUTH_OPTIONS,
        indent=0,
        auth_api=STREAMS_V1_LOCATION,
    )


K8S_LOCATION_OPTIONS = r"""
location {app} {{
    auth_request     {streams_api}k8s/auth/;
    auth_request_set $auth_status $upstream_status;
    auth_request_set $k8s_token $upstream_http_k8s_token;
    auth_request_set $k8s_uri $upstream_http_k8s_uri;
    proxy_pass $k8s_uri;
    proxy_http_version 1.1;
    proxy_redirect     off;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_hide_header X-Frame-Options;
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header Authorization "bearer $k8s_token";
    proxy_buffering off;
}}
"""  # noqa


def get_k8s_root_location_config():
    return get_config(
        options=K8S_LOCATION_OPTIONS,
        app=K8S_V1_LOCATION,
        streams_api=STREAMS_V1_LOCATION,
    )



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import uuid

from typing import Optional

from marshmallow import ValidationError

from polyaxon.contexts.params import PARAM_REGEX

OPS = "ops"
RUNS = "runs"
DAG = "dag"
DAG_ENTITY_REF = "_"
ENTITIES = {OPS, RUNS}

ENTITY_REF_FORMAT = "{}.{}"


def is_runs_ref(ref: str) -> bool:
    return ref.split(".")[0] == RUNS


def is_ops_ref(ref: str) -> bool:
    return ref.split(".")[0] == OPS


def is_dag_ref(ref: str) -> bool:
    return ref.split(".")[0] == DAG


def get_entity_ref(ref: str) -> Optional[str]:
    if is_ops_ref(ref) or is_runs_ref(ref):
        return ref.split(".")[1]
    if is_dag_ref(ref):
        return DAG_ENTITY_REF
    return None


def get_entity_type(value: str) -> str:
    value_parts = PARAM_REGEX.search(value)
    if value_parts:
        value_parts = value_parts.group(1).strip()
    else:
        value_parts = value

    return value_parts.split(".")[0]


def get_entity_value(value: str) -> str:
    value_parts = PARAM_REGEX.search(value)
    if value_parts:
        value_parts = value_parts.group(1).strip()
    else:
        value_parts = value

    value_parts = value_parts.split(".")
    if len(value_parts) < 2:
        return None
    return value_parts[-1]


def parse_ref_value(value: str) -> str:
    """Returns value without {{ }}"""
    value_parts = PARAM_REGEX.search(value)
    if value_parts:
        return value_parts.group(1).strip()
    return value


class RefMixin:
    @property
    def is_literal(self):
        return not self.ref

    @property
    def is_ref(self):
        return self.ref is not None

    @property
    def is_template_ref(self):
        try:
            value_parts = PARAM_REGEX.search(self.value)
            if value_parts:
                return True
        except Exception:  # noqa
            pass
        return False

    @property
    def is_runs_ref(self):
        if not self.is_ref:
            return False

        return is_runs_ref(self.ref)

    @property
    def is_ops_ref(self):
        if not self.is_ref:
            return False

        return is_ops_ref(self.ref)

    @property
    def is_dag_ref(self):
        if not self.is_ref:
            return False

        return is_dag_ref(self.ref)

    @property
    def is_join_ref(self):
        return False

    @property
    def entity_ref(self) -> Optional[str]:
        return get_entity_ref(self.ref)


def validate_ref(ref: str, name: str):
    # validate ref
    ref_parts = ref.split(".")
    if len(ref_parts) > 2:
        raise ValidationError(
            "Could not parse ref `{}` for param `{}`.".format(ref, name)
        )
    if len(ref_parts) == 1 and ref_parts[0] != DAG:
        raise ValidationError(
            "Could not parse ref `{}` for param `{}`.".format(ref_parts[0], name)
        )
    if len(ref_parts) == 2 and ref_parts[0] not in ENTITIES:
        raise ValidationError(
            "Could not parse ref `{}` for param `{}`. "
            "Operation ref must be one of `{}`".format(ref_parts[0], name, ENTITIES)
        )
    if ref_parts[0] == RUNS:
        try:
            uuid.UUID(ref_parts[1])
        except (KeyError, ValueError):
            raise ValidationError(
                "Param value `{}` should reference a valid run uuid.".format(
                    ref_parts[1]
                )
            )



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any, Dict, Iterable

from polyaxon.exceptions import PQLException
from polyaxon.pql.builder import QueryCondSpec
from polyaxon.pql.parser import parse_field, tokenize_query


class PQLManager:
    NAME = None
    FIELDS_USE_UUID = None
    FIELDS_USE_NAME = None
    FIELDS_PROXY = {}
    FIELDS_TRANS = {}
    FIELDS_ORDERING = None
    FIELDS_ORDERING_PROXY = None
    FIELDS_DEFAULT_ORDERING = None
    CHECK_ALIVE = True
    PARSERS_BY_FIELD = {}
    CONDITIONS_BY_FIELD = {}
    QUERY_BACKEND = None
    TIMEZONE = None

    @classmethod
    def proxy_field(cls, field: str) -> str:
        field, suffix = parse_field(field)
        if field in cls.FIELDS_PROXY:
            field = cls.FIELDS_PROXY[field]
        if cls.FIELDS_USE_UUID and not suffix and field in cls.FIELDS_USE_UUID:
            suffix = "uuid"
        if cls.FIELDS_USE_UUID and field in cls.FIELDS_USE_UUID and suffix == "id":
            suffix = "uuid"
        if cls.FIELDS_USE_NAME and not suffix and field in cls.FIELDS_USE_NAME:
            suffix = "name"
        if cls.FIELDS_USE_NAME and field in cls.FIELDS_USE_NAME and suffix == "id":
            suffix = "name"
        return "{}__{}".format(field, suffix) if suffix else field

    @classmethod
    def trans_field(
        cls,
        key: str,
        tokenized_query: Dict[str, Iterable],
        update_tokenized_query: Dict[str, Iterable],
    ) -> None:
        field, suffix = parse_field(key)
        if field in cls.FIELDS_TRANS:
            field_trans = cls.FIELDS_TRANS[field]["field"]
            update_tokenized_query["{}_value".format(field_trans)] = tokenized_query[
                key
            ]
            update_tokenized_query["{}_name".format(field_trans)] = [suffix]
            if cls.FIELDS_TRANS[field].get("type"):
                update_tokenized_query[
                    "{}_type".format(field_trans)
                ] = cls.FIELDS_TRANS[field]["type"]
        else:
            update_tokenized_query[key] = tokenized_query[key]

    @classmethod
    def tokenize(cls, query_spec: str) -> Dict[str, Iterable]:
        tokenized_query = tokenize_query(query_spec)
        results = {}
        for key in tokenized_query.keys():
            field, _ = parse_field(key)
            if field and (
                field not in cls.PARSERS_BY_FIELD
                or field not in cls.CONDITIONS_BY_FIELD
            ):
                raise PQLException(
                    "key `{}` is not supported by the query manager `{}`.".format(
                        key, cls.NAME
                    )
                )
            cls.trans_field(key, tokenized_query, results)
        return results

    @classmethod
    def parse(cls, tokenized_query: Dict[str, Iterable]) -> Dict[str, Iterable]:
        parsed_query = {}
        for key, expressions in tokenized_query.items():
            field, _ = parse_field(key)
            parsed_query[key] = [
                cls.PARSERS_BY_FIELD[field](exp) for exp in expressions
            ]
        return parsed_query

    @classmethod
    def build(cls, parsed_query: Dict[str, Iterable]) -> Dict[str, Iterable]:
        built_query = {}
        for key, operations in parsed_query.items():
            field, _ = parse_field(key)
            built_query[key] = [
                QueryCondSpec(
                    cond=cls.CONDITIONS_BY_FIELD[field](
                        op=op_spec.op, negation=op_spec.negation
                    ),
                    params=op_spec.params,
                )
                for op_spec in operations
            ]
        return built_query

    @classmethod
    def handle_query(cls, query_spec: str) -> Dict[str, Iterable]:
        tokenized_query = cls.tokenize(query_spec=query_spec)
        parsed_query = cls.parse(tokenized_query=tokenized_query)
        built_query = cls.build(parsed_query=parsed_query)
        return built_query

    @classmethod
    def apply(cls, query_spec: str, queryset: Any) -> Any:
        built_query = cls.handle_query(query_spec=query_spec)
        operators = []
        for key, cond_specs in built_query.items():
            key = cls.proxy_field(key)
            for cond_spec in cond_specs:
                try:
                    operator = cond_spec.cond.apply_operator(
                        name=key,
                        params=cond_spec.params,
                        query_backend=cls.QUERY_BACKEND,
                        timezone=cls.TIMEZONE,
                    )
                except Exception as e:
                    raise PQLException(
                        "Error applying operator for key `%s` by the query manager `%s`. "
                        "%s" % (key, cls.NAME, e)
                    )
                operators.append(operator)

        return queryset.filter(*operators)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
import os

from collections import namedtuple
from datetime import datetime
from typing import Dict, List

from marshmallow import fields

from polyaxon.containers.contexts import CONTEXT_MOUNT_FILE_WATCHER
from polyaxon.schemas.base import BaseConfig, BaseSchema
from polyaxon.utils.date_utils import path_last_modified
from polyaxon.utils.path_utils import get_files_and_dirs_in_path


class PathData(namedtuple("PathData", "base ts op")):
    pass


class FSWatcherSchema(BaseSchema):
    dir_mapping = fields.Dict(allow_none=True)
    file_mapping = fields.Dict(allow_none=True)

    @staticmethod
    def schema_config():
        return FSWatcherConfig


class FSWatcherConfig(BaseConfig):
    SCHEMA = FSWatcherSchema
    IDENTIFIER = "fswatcher"

    def __init__(self, dir_mapping=None, file_mapping=None, **kwargs):
        self.dir_mapping = dir_mapping
        self.file_mapping = file_mapping

    @staticmethod
    def _datetime_handler(value):
        if isinstance(value, datetime):
            return value.isoformat()
        raise TypeError("Unknown type")

    @classmethod
    def _dump(cls, obj_dict):
        return json.dumps(obj_dict, default=cls._datetime_handler)

    @staticmethod
    def _parse_mapping(mapping: Dict):
        if not mapping:
            return None
        return {
            k: PathData(v[0], datetime.fromisoformat(v[1]), v[2])
            for k, v in mapping.items()
        }

    @staticmethod
    def delete(path: str):
        if os.path.exists(path):
            os.remove(path)

    def get_dir_mapping(self):
        return self._parse_mapping(self.dir_mapping)

    def get_file_mapping(self):
        return self._parse_mapping(self.file_mapping)


class FSWatcher:
    PUT = "put"
    RM = "rm"
    NOOP = ""

    def __init__(self, dir_mapping: Dict = None, file_mapping: Dict = None):
        self._dir_mapping = dir_mapping or {}
        self._file_mapping = file_mapping or {}

    @classmethod
    def read(cls, config_path: str = CONTEXT_MOUNT_FILE_WATCHER):
        if not os.path.exists(config_path):
            return cls()
        config = FSWatcherConfig.read(config_path)
        return cls(
            dir_mapping=config.get_dir_mapping(), file_mapping=config.get_file_mapping()
        )

    def write(self, config_path: str = CONTEXT_MOUNT_FILE_WATCHER):
        config = FSWatcherConfig.read(
            {
                "dir_mapping": self._dir_mapping,
                "file_mapping": self._file_mapping,
            }
        )
        return config.write(config_path)

    def _sync_path(self, path: str, base_path: str, mapping: Dict):
        current_ts = path_last_modified(path)
        rel_path = os.path.relpath(path, base_path)
        data = mapping.get(rel_path)
        if data:
            if current_ts > data.ts:
                mapping[rel_path] = PathData(base_path, current_ts, self.PUT)
            else:
                mapping[rel_path] = PathData(base_path, data.ts, self.NOOP)
        else:
            mapping[rel_path] = PathData(base_path, current_ts, self.PUT)
        return mapping

    def sync_file(self, path: str, base_path: str):
        self._file_mapping = self._sync_path(path, base_path, self._file_mapping)

    def sync_dir(self, path: str, base_path: str):
        self._dir_mapping = self._sync_path(path, base_path, self._dir_mapping)

    def init(self):
        self._dir_mapping = {
            p: PathData(d.base, d.ts, self.RM) for p, d in self._dir_mapping.items()
        }
        self._file_mapping = {
            p: PathData(d.base, d.ts, self.RM) for p, d in self._file_mapping.items()
        }

    def sync(self, path: str, exclude: List[str] = None):
        files, dirs = get_files_and_dirs_in_path(
            path, exclude=exclude, collect_dirs=True
        )
        base_path, prefix_path = os.path.split(path)
        for file_path in files:
            self.sync_file(os.path.join(prefix_path, file_path), base_path=base_path)

        for dir_path in dirs:
            self.sync_dir(os.path.join(prefix_path, dir_path), base_path=base_path)

    def _get_mapping_by_op(self, mapping: Dict, op: str):
        return {(p.base, k) for k, p in mapping.items() if p.op == op}

    def _clean_by_op(self, mapping: Dict, op: str):
        return {k: p for k, p in mapping.items() if p.op != op}

    def get_files_to_put(self):
        return self._get_mapping_by_op(self._file_mapping, self.PUT)

    def get_files_to_rm(self):
        results = self._get_mapping_by_op(self._file_mapping, self.RM)
        self._file_mapping = self._clean_by_op(self._file_mapping, self.RM)
        return results

    def get_dirs_to_put(self):
        return self._get_mapping_by_op(self._dir_mapping, self.PUT)

    def get_dirs_to_rm(self):
        results = self._get_mapping_by_op(self._dir_mapping, self.RM)
        self._file_mapping = self._clean_by_op(self._file_mapping, self.RM)
        return results



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from typing import List, Optional, Union

from polyaxon.connections.reader import read_keys

logging.getLogger("azure").setLevel(logging.WARNING)
logging.getLogger("azure.storage").setLevel(logging.WARNING)
logging.getLogger("azure.storage.blob").setLevel(logging.WARNING)


def get_account_name(
    keys: Optional[Union[str, List[str]]] = None,
    context_path: Optional[str] = None,
    **kwargs
):
    # Check kwargs
    value = kwargs.get("account_name") or kwargs.get("AZURE_ACCOUNT_NAME")
    if value:
        return value
    # Check env/path keys
    keys = keys or ["AZURE_ACCOUNT_NAME"]
    return read_keys(context_path=context_path, keys=keys)


def get_account_key(
    keys: Optional[Union[str, List[str]]] = None,
    context_path: Optional[str] = None,
    **kwargs
):
    value = kwargs.get("account_key") or kwargs.get("AZURE_ACCOUNT_KEY")
    if value:
        return value
    keys = keys or ["AZURE_ACCOUNT_KEY"]
    return read_keys(context_path=context_path, keys=keys)


def get_connection_string(
    keys: Optional[Union[str, List[str]]] = None,
    context_path: Optional[str] = None,
    **kwargs
):
    value = kwargs.get("connection_string") or kwargs.get("AZURE_CONNECTION_STRING")
    if value:
        return value
    keys = keys or ["AZURE_CONNECTION_STRING"]
    return read_keys(context_path=context_path, keys=keys)


def get_sas_token(
    keys: Optional[Union[str, List[str]]] = None,
    context_path: Optional[str] = None,
    **kwargs
):
    value = (
        kwargs.get("sas_token")
        or kwargs.get("AZURE_SAS_TOKEN")
        or kwargs.get("AZURE_STORAGE_SAS_TOKEN")
    )
    if value:
        return value
    keys = keys or ["AZURE_SAS_TOKEN", "AZURE_STORAGE_SAS_TOKEN"]
    return read_keys(context_path=context_path, keys=keys)


def get_tenant_id(
    keys: Optional[Union[str, List[str]]] = None,
    context_path: Optional[str] = None,
    **kwargs
):
    value = kwargs.get("tenant_id") or kwargs.get("AZURE_TENANT_ID")
    if value:
        return value
    keys = keys or ["AZURE_TENANT_ID"]
    return read_keys(context_path=context_path, keys=keys)


def get_client_id(
    keys: Optional[Union[str, List[str]]] = None,
    context_path: Optional[str] = None,
    **kwargs
):
    value = kwargs.get("client_id") or kwargs.get("AZURE_CLIENT_ID")
    if value:
        return value
    keys = keys or ["AZURE_CLIENT_ID"]
    return read_keys(context_path=context_path, keys=keys)


def get_client_secret(
    keys: Optional[Union[str, List[str]]] = None,
    context_path: Optional[str] = None,
    **kwargs
):
    value = kwargs.get("client_secret") or kwargs.get("AZURE_CLIENT_SECRET")
    if value:
        return value
    keys = keys or ["AZURE_CLIENT_SECRET"]
    return read_keys(context_path=context_path, keys=keys)



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from datetime import datetime
from typing import Any, Dict, List, Optional, Sequence, Union

from polyaxon import settings
from polyaxon.client import RunClient
from polyaxon.client.decorators import client_handler
from polyaxon.polyboard.artifacts import V1ArtifactKind, V1RunArtifact
from polyaxon.tracking.run import Run

TRACKING_RUN = None


def init(
    owner: str = None,
    project: str = None,
    run_uuid: str = None,
    client: RunClient = None,
    track_code: bool = True,
    track_env: bool = True,
    refresh_data: bool = False,
    artifacts_path: str = None,
    collect_artifacts: str = None,
    collect_resources: str = None,
    is_offline: bool = None,
    is_new: bool = None,
    name: str = None,
    description: str = None,
    tags: List[str] = None,
):
    """Tracking module is similar to the tracking client without the need to create a run instance.

    The tracking module allows you to call all tracking methods directly from the top level module.

    This could be very convenient especially if you are running in-cluster experiments:

    ```python
    from polyaxon import tracking

    tracking.init()
    ...
    tracking.log_metrics(step=1, loss=0.09, accuracy=0.75)
    ...
    tracking.log_metrics(step=1, loss=0.02, accuracy=0.85)
    ...
    ```

    > A global `TRACKING_RUN` will be set on the module.


        Args:
            owner: str, optional, the owner is the username or
                 the organization name owning this project.
            project: str, optional, project name owning the run(s).
            run_uuid: str, optional, run uuid.
            client: [PolyaxonClient](/docs/core/python-library/polyaxon-client/), optional,
                 an instance of a configured client, if not passed,
                 a new instance will be created based on the available environment.
            track_code: bool, optional, default True, to track code version.
                 Polyaxon will try to track information about any repo
                 configured in the context where this client is instantiated.
            track_env: bool, optional, default True, to track information about the environment.
            refresh_data: bool, optional, default False, to refresh the run data at instantiation.
            refresh_data: bool, optional, default False, to instruct the run to resume,
                 only useful when the run is not managed by Polyaxon.
            artifacts_path: str, optional, for in-cluster runs it will be set automatically.
            collect_artifacts: bool, optional,
                 similar to the env var flag `POLYAXON_COLLECT_ARTIFACTS`, this env var is `True`
                 by default for managed runs and is controlled by the plugins section.
            collect_resources: bool, optional,
                 similar to the env var flag `POLYAXON_COLLECT_RESOURCES`, this env var is `True`
                 by default for managed runs and is controlled by the plugins section.
            is_offline: bool, optional,
                 To trigger the offline mode manually instead of depending on `POLYAXON_IS_OFFLINE`.
            is_new: bool, optional,
                 Force the creation of a new run instead of trying to discover a cached run or
                 refreshing an instance from the env var
            name: str, optional,
                 When `is_new` or `is_offline` is set to true, a new instance is created and
                 you can initialize that new run with a name.
            description: str, optional,
                 When `is_new` or `is_offline` is set to true, a new instance is created and
                 you can initialize that new run with a description.
            tags: str or List[str], optional,
                 When `is_new` or `is_offline` is set to true, a new instance is created and
                 you can initialize that new run with tags.

        Raises:
            PolyaxonClientException: If no owner and/or project are passed and Polyaxon cannot
                 resolve the values from the environment.
    """
    global TRACKING_RUN

    TRACKING_RUN = Run(
        owner=owner,
        project=project,
        run_uuid=run_uuid,
        client=client,
        track_code=track_code,
        refresh_data=refresh_data,
        track_env=track_env,
        artifacts_path=artifacts_path,
        collect_artifacts=collect_artifacts,
        collect_resources=collect_resources,
        is_offline=is_offline,
        is_new=is_new,
        name=name,
        description=description,
        tags=tags,
    )


@client_handler(check_no_op=True)
def get_or_create_run(tracking_run: Run = None) -> Optional[Run]:
    """Get or create a new tracking run.

    It tries to create a new instance, for in-cluster runs, this will work automatically.

    This is used inside some Polyaxon callbacks, you should use `init` instead.
    """
    global TRACKING_RUN

    if tracking_run:
        return tracking_run
    if TRACKING_RUN:
        return TRACKING_RUN

    if settings.CLIENT_CONFIG.is_managed:
        init()
        return TRACKING_RUN


def get_artifacts_path(
    rel_path: str = None,
    ensure_path: bool = False,
    is_dir: bool = False,
    use_store_path: bool = False,
):
    global TRACKING_RUN
    return TRACKING_RUN.get_artifacts_path(
        rel_path=rel_path,
        ensure_path=ensure_path,
        is_dir=is_dir,
        use_store_path=use_store_path,
    )


get_artifacts_path.__doc__ = Run.get_artifacts_path.__doc__


def get_outputs_path(
    rel_path: str = None,
    ensure_path: bool = True,
    is_dir: bool = False,
    use_store_path: bool = False,
):
    global TRACKING_RUN
    return TRACKING_RUN.get_outputs_path(
        rel_path=rel_path,
        ensure_path=ensure_path,
        is_dir=is_dir,
        use_store_path=use_store_path,
    )


get_outputs_path.__doc__ = Run.get_outputs_path.__doc__


def get_tensorboard_path(
    rel_path: str = "tensorboard",
    use_store_path: bool = False,
):
    global TRACKING_RUN
    return TRACKING_RUN.get_tensorboard_path(
        rel_path=rel_path, use_store_path=use_store_path
    )


get_tensorboard_path.__doc__ = Run.get_tensorboard_path.__doc__


def set_artifacts_path(artifacts_path: str, is_related: bool = False):
    global TRACKING_RUN
    TRACKING_RUN.set_artifacts_path(artifacts_path, is_related)


set_artifacts_path.__doc__ = Run.set_artifacts_path.__doc__


def set_run_event_logger():
    global TRACKING_RUN
    TRACKING_RUN.set_run_event_logger()


set_run_event_logger.__doc__ = Run.set_run_event_logger.__doc__


def set_run_resource_logger():
    global TRACKING_RUN
    TRACKING_RUN.set_run_resource_logger()


set_run_resource_logger.__doc__ = Run.set_run_resource_logger.__doc__


def set_run_process_sidecar():
    global TRACKING_RUN
    TRACKING_RUN.set_run_process_sidecar()


set_run_process_sidecar.__doc__ = Run.set_run_process_sidecar.__doc__


def log_metric(name: str, value: float, step: int = None, timestamp: datetime = None):
    global TRACKING_RUN
    TRACKING_RUN.log_metric(
        name=name,
        value=value,
        step=step,
        timestamp=timestamp,
    )


log_metric.__doc__ = Run.log_metric.__doc__


def log_metrics(step: int = None, timestamp: datetime = None, **metrics):
    global TRACKING_RUN
    TRACKING_RUN.log_metrics(step=step, timestamp=timestamp, **metrics)


log_metrics.__doc__ = Run.log_metrics.__doc__


def log_roc_auc_curve(
    name: str, fpr, tpr, auc: float = None, step: int = None, timestamp: datetime = None
):
    global TRACKING_RUN
    TRACKING_RUN.log_roc_auc_curve(
        name=name,
        fpr=fpr,
        tpr=tpr,
        auc=auc,
        step=step,
        timestamp=timestamp,
    )


log_roc_auc_curve.__doc__ = Run.log_roc_auc_curve.__doc__


def log_sklearn_roc_auc_curve(
    name: str,
    y_preds,
    y_targets,
    step: int = None,
    timestamp: datetime = None,
    is_multi_class: bool = False,
):
    global TRACKING_RUN
    TRACKING_RUN.log_sklearn_roc_auc_curve(
        name=name,
        y_preds=y_preds,
        y_targets=y_targets,
        step=step,
        timestamp=timestamp,
        is_multi_class=is_multi_class,
    )


log_sklearn_roc_auc_curve.__doc__ = Run.log_sklearn_roc_auc_curve.__doc__


def log_pr_curve(
    name: str,
    precision,
    recall,
    average_precision=None,
    step: int = None,
    timestamp: datetime = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_pr_curve(
        name=name,
        precision=precision,
        recall=recall,
        average_precision=average_precision,
        step=step,
        timestamp=timestamp,
    )


log_pr_curve.__doc__ = Run.log_pr_curve.__doc__


def log_sklearn_pr_curve(
    name: str,
    y_preds,
    y_targets,
    step: int = None,
    timestamp: datetime = None,
    is_multi_class: bool = False,
):
    global TRACKING_RUN
    TRACKING_RUN.log_sklearn_pr_curve(
        name=name,
        y_preds=y_preds,
        y_targets=y_targets,
        step=step,
        timestamp=timestamp,
        is_multi_class=is_multi_class,
    )


log_sklearn_pr_curve.__doc__ = Run.log_sklearn_pr_curve.__doc__


def log_curve(
    name: str,
    x,
    y,
    annotation: str = None,
    step: int = None,
    timestamp: datetime = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_curve(
        name=name,
        x=x,
        y=y,
        annotation=annotation,
        step=step,
        timestamp=timestamp,
    )


log_curve.__doc__ = Run.log_curve.__doc__


def log_confusion_matrix(
    name: str,
    x,
    y,
    z: str = None,
    step: int = None,
    timestamp: datetime = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_confusion_matrix(
        name=name,
        x=x,
        y=y,
        z=z,
        step=step,
        timestamp=timestamp,
    )


log_confusion_matrix.__doc__ = Run.log_confusion_matrix.__doc__


def log_image(
    data: Any,
    name: str = None,
    step: int = None,
    timestamp: datetime = None,
    rescale=1,
    dataformats: str = "CHW",
    ext: str = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_image(
        data=data,
        name=name,
        step=step,
        timestamp=timestamp,
        rescale=rescale,
        dataformats=dataformats,
        ext=ext,
    )


log_image.__doc__ = Run.log_image.__doc__


def log_image_with_boxes(
    tensor_image,
    tensor_boxes,
    name: str = None,
    step: int = None,
    timestamp: datetime = None,
    rescale: int = 1,
    dataformats: str = "CHW",
):
    global TRACKING_RUN
    TRACKING_RUN.log_image_with_boxes(
        tensor_image=tensor_image,
        tensor_boxes=tensor_boxes,
        name=name,
        step=step,
        timestamp=timestamp,
        rescale=rescale,
        dataformats=dataformats,
    )


log_image_with_boxes.__doc__ = Run.log_image_with_boxes.__doc__


def log_mpl_image(
    data,
    name: str = None,
    close: bool = True,
    step: int = None,
    timestamp: datetime = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_mpl_image(
        data=data,
        name=name,
        close=close,
        step=step,
        timestamp=timestamp,
    )


log_mpl_image.__doc__ = Run.log_mpl_image.__doc__


def log_video(
    data,
    name: str = None,
    fps: int = 4,
    step: int = None,
    timestamp: datetime = None,
    content_type: str = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_video(
        data=data,
        name=name,
        fps=fps,
        step=step,
        timestamp=timestamp,
        content_type=content_type,
    )


log_video.__doc__ = Run.log_video.__doc__


def log_audio(
    data,
    name: str = None,
    sample_rate: int = 44100,
    step: int = None,
    timestamp: datetime = None,
    content_type: str = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_audio(
        data=data,
        name=name,
        sample_rate=sample_rate,
        step=step,
        timestamp=timestamp,
        content_type=content_type,
    )


log_audio.__doc__ = Run.log_audio.__doc__


def log_text(name: str, text: str, step: int = None, timestamp: datetime = None):
    global TRACKING_RUN
    TRACKING_RUN.log_text(
        name=name,
        text=text,
        step=step,
        timestamp=timestamp,
    )


log_text.__doc__ = Run.log_text.__doc__


def log_html(name: str, html: str, step: int = None, timestamp: datetime = None):
    global TRACKING_RUN
    TRACKING_RUN.log_html(
        name=name,
        html=html,
        step=step,
        timestamp=timestamp,
    )


log_html.__doc__ = Run.log_html.__doc__


def log_np_histogram(
    name: str, values, counts, step: int = None, timestamp: datetime = None
):
    global TRACKING_RUN
    TRACKING_RUN.log_np_histogram(
        name=name,
        values=values,
        counts=counts,
        step=step,
        timestamp=timestamp,
    )


log_np_histogram.__doc__ = Run.log_np_histogram.__doc__


def log_histogram(
    name: str, values, bins, max_bins=None, step: int = None, timestamp: datetime = None
):
    global TRACKING_RUN
    TRACKING_RUN.log_histogram(
        name=name,
        values=values,
        bins=bins,
        max_bins=max_bins,
        step=step,
        timestamp=timestamp,
    )


log_histogram.__doc__ = Run.log_histogram.__doc__


def log_model(
    path: str,
    name: str = None,
    framework: str = None,
    summary: Dict = None,
    step: int = None,
    timestamp: datetime = None,
    rel_path: str = "model",
    versioned: bool = True,
):
    global TRACKING_RUN
    TRACKING_RUN.log_model(
        path=path,
        name=name,
        framework=framework,
        summary=summary,
        step=step,
        timestamp=timestamp,
        rel_path=rel_path,
        versioned=versioned,
    )


log_model.__doc__ = Run.log_model.__doc__


def log_artifact(
    path: str,
    name: str = None,
    kind: str = None,
    summary: Dict = None,
    step: int = None,
    timestamp: datetime = None,
    rel_path: str = None,
    versioned: bool = True,
    **kwargs
):
    global TRACKING_RUN
    TRACKING_RUN.log_artifact(
        path=path,
        name=name,
        kind=kind,
        summary=summary,
        step=step,
        timestamp=timestamp,
        rel_path=rel_path,
        versioned=versioned,
        **kwargs,
    )


log_artifact.__doc__ = Run.log_artifact.__doc__


def log_dataframe(
    df,
    name: str,
    content_type: str = V1ArtifactKind.CSV,
    step: int = None,
    timestamp: datetime = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_dataframe(
        df=df,
        name=name,
        content_type=content_type,
        step=step,
        timestamp=timestamp,
    )


log_dataframe.__doc__ = Run.log_dataframe.__doc__


def log_plotly_chart(name: str, figure, step: int = None, timestamp: datetime = None):
    global TRACKING_RUN
    TRACKING_RUN.log_plotly_chart(
        name=name,
        figure=figure,
        step=step,
        timestamp=timestamp,
    )


log_plotly_chart.__doc__ = Run.log_plotly_chart.__doc__


def log_bokeh_chart(name: str, figure, step: int = None, timestamp: datetime = None):
    global TRACKING_RUN
    TRACKING_RUN.log_bokeh_chart(
        name=name,
        figure=figure,
        step=step,
        timestamp=timestamp,
    )


log_bokeh_chart.__doc__ = Run.log_bokeh_chart.__doc__


def log_altair_chart(name: str, figure, step: int = None, timestamp: datetime = None):
    global TRACKING_RUN
    TRACKING_RUN.log_altair_chart(
        name=name,
        figure=figure,
        step=step,
        timestamp=timestamp,
    )


log_altair_chart.__doc__ = Run.log_altair_chart.__doc__


def log_mpl_plotly_chart(
    name: str, figure, step: int = None, timestamp: datetime = None
):
    global TRACKING_RUN
    TRACKING_RUN.log_mpl_plotly_chart(
        name=name,
        figure=figure,
        step=step,
        timestamp=timestamp,
    )


log_mpl_plotly_chart.__doc__ = Run.log_mpl_plotly_chart.__doc__


def get_log_level():
    global TRACKING_RUN
    return TRACKING_RUN.get_log_level()


get_log_level.__doc__ = Run.get_log_level.__doc__


def set_description(description: str, async_req: bool = True):
    global TRACKING_RUN
    TRACKING_RUN.set_description(description=description, async_req=async_req)


set_description.__doc__ = Run.set_description.__doc__


def set_name(name: str, async_req: bool = True):
    global TRACKING_RUN
    TRACKING_RUN.set_name(name=name, async_req=async_req)


set_name.__doc__ = Run.set_name.__doc__


def end():
    global TRACKING_RUN
    TRACKING_RUN.end()


end.__doc__ = Run.end.__doc__


def log_status(
    status: str,
    reason: str = None,
    message: str = None,
    last_transition_time: datetime = None,
    last_update_time: datetime = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_status(
        status=status,
        reason=reason,
        message=message,
        last_transition_time=last_transition_time,
        last_update_time=last_update_time,
    )


log_status.__doc__ = Run.log_status.__doc__


def log_inputs(reset: bool = False, async_req: bool = True, **inputs):
    global TRACKING_RUN
    TRACKING_RUN.log_inputs(reset=reset, async_req=async_req, **inputs)


log_inputs.__doc__ = Run.log_inputs.__doc__


def log_outputs(reset: bool = False, async_req: bool = True, **outputs):
    global TRACKING_RUN
    TRACKING_RUN.log_outputs(reset=reset, async_req=async_req, **outputs)


log_outputs.__doc__ = Run.log_outputs.__doc__


def log_tags(
    tags: Union[str, Sequence[str]],
    reset: bool = False,
    async_req: bool = True,
):
    global TRACKING_RUN
    TRACKING_RUN.log_tags(tags=tags, reset=reset, async_req=async_req)


log_tags.__doc__ = Run.log_tags.__doc__


def log_meta(reset: bool = False, async_req: bool = True, **meta):
    global TRACKING_RUN
    TRACKING_RUN.log_meta(reset=reset, async_req=async_req, **meta)


log_meta.__doc__ = Run.log_meta.__doc__


def log_progress(value: float):
    global TRACKING_RUN
    TRACKING_RUN.log_progress(value=value)


log_progress.__doc__ = Run.log_progress.__doc__


def log_succeeded():
    global TRACKING_RUN
    TRACKING_RUN.log_succeeded()


log_succeeded.__doc__ = Run.log_succeeded.__doc__


def log_stopped():
    global TRACKING_RUN
    TRACKING_RUN.log_stopped()


log_stopped.__doc__ = Run.log_stopped.__doc__


def log_failed(reason: str = None, message: str = None):
    global TRACKING_RUN
    TRACKING_RUN.log_failed(reason=reason, message=message)


log_failed.__doc__ = Run.log_failed.__doc__


def log_artifact_ref(
    path: str,
    kind: V1ArtifactKind,
    name: str = None,
    hash: str = None,
    content=None,
    summary: Dict = None,
    is_input: bool = False,
    rel_path: str = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_artifact_ref(
        path=path,
        kind=kind,
        name=name,
        hash=hash,
        content=content,
        summary=summary,
        is_input=is_input,
        rel_path=rel_path,
    )


log_artifact_ref.__doc__ = Run.log_artifact_ref.__doc__


def log_model_ref(
    path: str,
    name: str = None,
    framework: str = None,
    summary: Dict = None,
    is_input: bool = False,
    rel_path: str = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_model_ref(
        path=path,
        name=name,
        framework=framework,
        summary=summary,
        is_input=is_input,
        rel_path=rel_path,
    )


log_model_ref.__doc__ = Run.log_model_ref.__doc__


def log_code_ref(code_ref: Dict = None, is_input: bool = True):
    global TRACKING_RUN
    TRACKING_RUN.log_code_ref(code_ref=code_ref, is_input=is_input)


log_code_ref.__doc__ = Run.log_code_ref.__doc__


def log_data_ref(
    name: str,
    hash: str = None,
    path: str = None,
    content=None,
    summary: Dict = None,
    is_input: bool = True,
):
    global TRACKING_RUN
    TRACKING_RUN.log_data_ref(
        name=name,
        content=content,
        hash=hash,
        path=path,
        summary=summary,
        is_input=is_input,
    )


log_data_ref.__doc__ = Run.log_data_ref.__doc__


def log_file_ref(
    path: str,
    name: str = None,
    hash: str = None,
    content=None,
    summary: Dict = None,
    is_input: bool = True,
    rel_path: str = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_file_ref(
        path=path,
        name=name,
        hash=hash,
        content=content,
        summary=summary,
        is_input=is_input,
        rel_path=rel_path,
    )


log_file_ref.__doc__ = Run.log_file_ref.__doc__


def log_dir_ref(
    path: str,
    name: str = None,
    summary: Dict = None,
    is_input: bool = False,
    rel_path: str = None,
):
    global TRACKING_RUN
    TRACKING_RUN.log_dir_ref(
        path=path, name=name, summary=summary, is_input=is_input, rel_path=rel_path
    )


log_dir_ref.__doc__ = Run.log_dir_ref.__doc__


def log_artifact_lineage(body: List[V1RunArtifact]):
    global TRACKING_RUN
    TRACKING_RUN.log_artifact_lineage(body)


log_artifact_lineage.__doc__ = Run.log_artifact_lineage.__doc__


def log_env(rel_path: str = None, content: Dict = None):
    global TRACKING_RUN
    return TRACKING_RUN.log_env(rel_path=rel_path, content=content)


log_env.__doc__ = Run.log_env.__doc__


def sync_events_summaries():
    global TRACKING_RUN
    TRACKING_RUN.sync_events_summaries()


sync_events_summaries.__doc__ = Run.sync_events_summaries.__doc__


def sync_system_events_summaries():
    global TRACKING_RUN
    TRACKING_RUN.sync_system_events_summaries()


sync_system_events_summaries.__doc__ = Run.sync_system_events_summaries.__doc__


def sync_offline_run(
    artifacts_path: str = None,
    load_offline_run: bool = False,
):
    global TRACKING_RUN
    TRACKING_RUN.sync_offline_run(
        artifacts_path=artifacts_path,
        load_offline_run=load_offline_run,
    )


sync_offline_run.__doc__ = Run.sync_offline_run.__doc__



#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from datetime import datetime
from typing import List

import ujson

from starlette import status
from starlette.concurrency import run_in_threadpool
from starlette.exceptions import HTTPException

from polyaxon import settings
from polyaxon.fs.async_manager import (
    delete_file_or_dir,
    download_dir,
    open_file,
    upload_data,
)
from polyaxon.fs.types import FSSystem
from polyaxon.polyboard.logging import V1Log, V1Logs
from polyaxon.utils.path_utils import delete_path


async def clean_tmp_logs(fs: FSSystem, run_uuid: str):
    if not settings.AGENT_CONFIG.artifacts_store:
        raise HTTPException(
            detail="Run's logs was not collected, resource was not found.",
            status_code=status.HTTP_400_BAD_REQUEST,
        )
    subpath = "{}/.tmpplxlogs".format(run_uuid)
    delete_path(subpath)
    await delete_file_or_dir(fs=fs, subpath=subpath, is_file=False)


async def upload_logs(fs: FSSystem, run_uuid: str, logs: List[V1Log]):
    if not settings.AGENT_CONFIG.artifacts_store:
        raise HTTPException(
            detail="Run's logs was not collected, resource was not found.",
            status_code=status.HTTP_400_BAD_REQUEST,
        )
    for c_logs in V1Logs.chunk_logs(logs):
        last_file = datetime.timestamp(c_logs.logs[-1].timestamp)
        if settings.AGENT_CONFIG.compressed_logs:
            subpath = "{}/plxlogs/{}.plx".format(run_uuid, last_file)
            await upload_data(
                fs=fs,
                subpath=subpath,
                data="{}\n{}".format(c_logs.get_csv_header(), c_logs.to_csv()),
            )
        else:
            subpath = "{}/plxlogs/{}".format(run_uuid, last_file)
            await upload_data(fs=fs, subpath=subpath, data=c_logs.to_dict(dump=True))


async def content_to_logs(content, logs_path):
    if not content:
        return []

    def convert():
        # Version handling
        if ".plx" in logs_path:
            return V1Logs.read_csv(content).logs
        # Legacy logs
        return ujson.loads(content).get("logs", [])

    return await run_in_threadpool(convert)


async def download_logs_file(
    fs: FSSystem, run_uuid: str, last_file: str, check_cache: bool = True
) -> (str, str):
    subpath = "{}/plxlogs/{}".format(run_uuid, last_file)
    content = await open_file(fs=fs, subpath=subpath, check_cache=check_cache)

    return await content_to_logs(content, subpath)


async def download_tmp_logs(fs: FSSystem, run_uuid: str) -> str:
    subpath = "{}/.tmpplxlogs".format(run_uuid)
    delete_path(subpath)
    return await download_dir(fs=fs, subpath=subpath)
